@@ -1,7 +1,7 @@
 //************************************************************************//
 // Copyright (C) 2011-2012 Mikiya Fujii                                   // 
 // Copyright (C) 2012-2012 Katsuhiko Nishimra                             // 
-// Copyright (C) 2012-2012 Michihiro Okuyama                              // 
+// Copyright (C) 2012-2013 Michihiro Okuyama                              //
 //                                                                        // 
 // This file is part of MolDS.                                            // 
 //                                                                        // 
@@ -28,13 +28,17 @@
 #include<vector>
 #include<stdexcept>
 #include<boost/format.hpp>
+#include"Enums.h"
+#include"Uncopyable.h"
 #include"PrintController.h"
 #include"MolDSException.h"
-#include"Uncopyable.h"
+#include"MallocerFreer.h"
+#include"../mpi/MpiInt.h"
+#include"../mpi/MpiProcess.h"
 #include"Utilities.h"
-#include"Enums.h"
 #include"EularAngle.h"
 #include"Parameters.h"
+#include"RealSphericalHarmonicsIndex.h"
 #include"atoms/Atom.h"
 #include"atoms/Hatom.h"
 #include"atoms/Liatom.h"
@@ -78,10 +82,16 @@ void InputParser::SetMessages(){
       = "Error in base::InputParser::GetInputTerms: Input file is empty.\n"; 
    this->errorMessageNotFoundInputFile
       = "Error in base::InputParser::StoreInputTermsFromFile: Not found.\n"; 
+   this->errorMessageNonValidTheoriesEpc
+      = "Error in base::InputParser::ValidateEpcConditions: Theory you set is not supported for EPC. The supported theories are MNDO-sefies(MNDO, AM1, AM1D, PM3, PM3D, PDDG/PM3) only.\n";
+   this->errorMessageNonValidTheoriesMD
+      = "Error in base::InputParser::ValidateMdConditions: Theory you set is not supported for MD.\n";
    this->errorMessageNonValidExcitedStatesMD
       = "Error in base::InputParser::ValidateMdConditions: Excited state on which MD runs or CIS condition are wrong.\n";
    this->errorMessageNonValidExcitedStatesMC
       = "Error in base::InputParser::ValidateMcConditions: Excited state on which MC runs or CIS condition are wrong.\n";
+   this->errorMessageNonValidTheoriesRPMD
+      = "Error in base::InputParser::ValidateRpmdConditions: heory you set is not supported for RMPD.\n";
    this->errorMessageNonValidExcitedStatesRPMD
       = "Error in base::InputParser::ValidateRpmdConditions: Excited state on which RPMD runs or CIS condition are wrong.\n";
    this->errorMessageNonValidTheoriesNASCO
@@ -90,6 +100,8 @@ void InputParser::SetMessages(){
       = "Error in base::InputParser::ValidateNascoConditions: The Number of electronic states of NASCO should be not over the number of CIS excited states plus 1.\n";
    this->errorMessageNonValidInitialElectronicStateNASCO
       = "Error in base::InputParser::ValidateNascoConditions: The initial electronic states for NASCO should be set to one of the electronic eigenstates used in NASCO.\n";
+   this->errorMessageNonValidTheoriesOptimization
+      = "Error in base::InputParser::ValidateOptimizationConditions: heory you set is not supported for optimization.\n";
    this->errorMessageNonValidExcitedStatesOptimization
       = "Error in base::InputParser::ValidateOptimizationConditions: Excited state on which optimization is carried out or CIS condition are wrong.\n";
    this->errorMessageNonValidElectronicStateFrequencies
@@ -134,6 +146,7 @@ void InputParser::SetMessages(){
    this->messageCisExcitonEnergies            = "\t\tExciton energies: ";
    this->messageCisAllTransitionDipoleMoments = "\t\tAll transition dipole moments: ";
    this->messageCisNumPrintCoefficients       = "\t\tNumber of printed coefficients of CIS-eigenvector: ";
+   this->messageCisMulliken                   = "\t\tMulliken population of excited states: ";
 
    // memory
    this->messageMemoryConditions = "\tMemory conditions:\n";
@@ -237,6 +250,11 @@ void InputParser::SetMessages(){
    this->stringGeometry    = "geometry";
    this->stringGeometryEnd = "geometry_end";
 
+   // Environmental Point Charge
+   this->stringEpc       = "epc";
+   this->stringEpcEnd    = "epc_end";
+   this->stringEpcCharge = "charge";
+
    // SCF
    this->stringScf                 = "scf";
    this->stringScfEnd              = "scf_end";
@@ -309,6 +327,8 @@ void InputParser::SetMessages(){
    this->stringCISExcitonEnergies            = "exciton_energies";
    this->stringCISAllTransitionDipoleMoments = "all_transition_dipole_moments";
    this->stringCISNumPrintCoefficients       = "num_print_coefficients";
+   this->stringCISMulliken                   = "mulliken";
+   this->stringCISUnpairedPop                = "unpaired_electron_population";
 
    // Memory
    this->stringMemory          = "memory";
@@ -356,6 +376,7 @@ void InputParser::SetMessages(){
    this->stringOptimizationEnd               = "optimization_end";
    this->stringOptimizationMethod            = "method";
    this->stringOptimizationBFGS              = "bfgs";
+   this->stringOptimizationGEDIIS            = "gediis";
    this->stringOptimizationConjugateGradient = "conjugate_gradient";
    this->stringOptimizationSteepestDescent   = "steepest_descent";
    this->stringOptimizationTotalSteps        = "total_steps";
@@ -466,13 +487,34 @@ int InputParser::ParseMolecularGeometry(Molecule* molecule, vector<string>* inpu
       else if((*inputTerms)[parseIndex] == "s"){
          atomType = S;
       }
-      Atom* atom = AtomFactory::Create(atomType, x, y, z);
+      int index = molecule->GetNumberAtoms();
+      Atom* atom = AtomFactory::Create(atomType, index, x, y, z);
       molecule->AddAtom(atom);
       parseIndex += 4;
    }
    return parseIndex;
 }
 
+int InputParser::ParseEpcsConfiguration(Molecule* molecule, vector<string>* inputTerms, int parseIndex) const{
+   parseIndex++;
+   while((*inputTerms)[parseIndex].compare(this->stringEpcEnd) != 0){
+      double x      = atof((*inputTerms)[parseIndex+0].c_str()) * Parameters::GetInstance()->GetAngstrom2AU();
+      double y      = atof((*inputTerms)[parseIndex+1].c_str()) * Parameters::GetInstance()->GetAngstrom2AU();
+      double z      = atof((*inputTerms)[parseIndex+2].c_str()) * Parameters::GetInstance()->GetAngstrom2AU();
+      parseIndex += 3;
+      double charge = 0.0;
+      if((*inputTerms)[parseIndex].compare(this->stringEpcCharge) == 0){
+         charge = atof((*inputTerms)[parseIndex+1].c_str());
+         parseIndex += 2;
+      }
+      AtomType atomType = EPC;
+      int index = molecule->GetNumberEpcs();
+      Atom* atom = AtomFactory::Create(atomType, index, x, y, z, charge);
+      molecule->AddEpc(atom);
+   }
+   return parseIndex;
+}
+
 int InputParser::ParseConditionsSCF(vector<string>* inputTerms, int parseIndex) const{
    parseIndex++;
    while((*inputTerms)[parseIndex].compare(this->stringScfEnd) != 0){
@@ -812,6 +854,25 @@ int InputParser::ParseConditionsCIS(vector<string>* inputTerms, int parseIndex)
          }
          parseIndex++;
       }
+      // mulliken
+      if((*inputTerms)[parseIndex].compare(this->stringCISMulliken) == 0){
+         int groundStateIndex = 0;
+         int elecStateIndex = atoi((*inputTerms)[parseIndex+1].c_str());
+         if(groundStateIndex<elecStateIndex){
+            Parameters::GetInstance()->AddElectronicStateIndexMullikenCIS(elecStateIndex);
+         }
+         parseIndex++;
+      }
+      // unpaired electron population
+      if((*inputTerms)[parseIndex].compare(this->stringCISUnpairedPop) == 0){
+          if((*inputTerms)[parseIndex+1].compare(this->stringYES) == 0){
+             Parameters::GetInstance()->SetRequiresUnpairedPopCIS(true);
+          }
+          else {
+             Parameters::GetInstance()->SetRequiresUnpairedPopCIS(false);
+          }
+          parseIndex++;   
+      }
       parseIndex++;   
    }
    return parseIndex;
@@ -988,6 +1049,9 @@ int InputParser::ParseConditionsOptimization(vector<string>* inputTerms, int par
          else if((*inputTerms)[parseIndex+1].compare(this->stringOptimizationBFGS) == 0){
             Parameters::GetInstance()->SetMethodOptimization(BFGSMethod);
          }
+         else if((*inputTerms)[parseIndex+1].compare(this->stringOptimizationGEDIIS) == 0){
+            Parameters::GetInstance()->SetMethodOptimization(GEDIISMethod);
+         }
          else{
          }
          parseIndex++;
@@ -1132,6 +1196,11 @@ void InputParser::Parse(Molecule* molecule, int argc, char *argv[]) const{
          i = this->ParseMolecularGeometry(molecule, &inputTerms, i);
       }
 
+      // Environmental Point Charges Configuration(EPC)
+      if(inputTerms[i].compare(this->stringEpc) == 0){
+         i = this->ParseEpcsConfiguration(molecule, &inputTerms, i);
+      }
+
       // scf condition
       if(inputTerms[i].compare(this->stringScf) == 0){
          i = this->ParseConditionsSCF(&inputTerms, i);
@@ -1212,6 +1281,7 @@ void InputParser::Parse(Molecule* molecule, int argc, char *argv[]) const{
    // calculate basics and validate conditions
    this->CalcMolecularBasics(molecule);
    this->ValidateVdWConditions();
+   this->ValidateEpcConditions(*molecule);
    if(Parameters::GetInstance()->RequiresCIS()){
       this->ValidateCisConditions(*molecule);
    }
@@ -1289,6 +1359,25 @@ void InputParser::ValidateVdWConditions() const{
    }
 }
 
+void InputParser::ValidateEpcConditions(const Molecule& molecule) const{
+   if(molecule.GetNumberEpcs()<=0){return;}
+   TheoryType theory = Parameters::GetInstance()->GetCurrentTheory();
+   // Validate theory
+   if(theory == MNDO || 
+      theory == AM1  || 
+      theory == AM1D || 
+      theory == PM3  || 
+      theory == PM3D || 
+      theory == PM3PDDG ){
+   }
+   else{
+      stringstream ss;
+      ss << this->errorMessageNonValidTheoriesEpc;
+      ss << this->errorMessageTheory << TheoryTypeStr(theory) << endl;
+      throw MolDSException(ss.str());
+   }
+}
+
 void InputParser::ValidateCisConditions(const Molecule& molecule) const{
 
    // direct CIS
@@ -1326,6 +1415,20 @@ void InputParser::ValidateCisConditions(const Molecule& molecule) const{
       Parameters::GetInstance()->SetNumberPrintCoefficientsCIS(numberSlaterDeterminants);
    }   
 
+   // Validate electronic state for Mulliken population analysis
+   if(Parameters::GetInstance()->RequiresMullikenCIS()){
+      vector<int>* indecesMulliken = Parameters::GetInstance()->GetElectronicStateIndecesMullikenCIS();
+      int numExcitedStates = Parameters::GetInstance()->GetNumberExcitedStatesCIS();
+      vector<int>::iterator it=(*indecesMulliken).begin();
+      vector<int>::iterator end=(*indecesMulliken).end();
+      while(it<end){
+         if(numExcitedStates<*it){
+            it  = (*indecesMulliken).erase(it);
+            end = (*indecesMulliken).end();
+         }
+         ++it;
+      }
+   }
 }
 
 void InputParser::ValidateMdConditions(const Molecule& molecule) const{
@@ -1333,10 +1436,9 @@ void InputParser::ValidateMdConditions(const Molecule& molecule) const{
    int targetStateIndex = Parameters::GetInstance()->GetElectronicStateIndexMD();
    TheoryType theory = Parameters::GetInstance()->GetCurrentTheory();
    // Validate theory
-   if(theory == CNDO2 || theory == INDO || (theory == ZINDOS && groundStateIndex < targetStateIndex)){
+   if(theory == CNDO2 || theory == INDO ){
       stringstream ss;
-      ss << this->errorMessageNonValidExcitedStatesMD;
-      ss << this->errorMessageElecState << targetStateIndex << endl;
+      ss << this->errorMessageNonValidTheoriesMD;
       ss << this->errorMessageTheory << TheoryTypeStr(theory) << endl;
       throw MolDSException(ss.str());
    }
@@ -1347,7 +1449,7 @@ void InputParser::ValidateMdConditions(const Molecule& molecule) const{
       this->ValidateCisConditions(molecule);
    }
    int numberExcitedStatesCIS = Parameters::GetInstance()->GetNumberExcitedStatesCIS();
-   if(numberExcitedStatesCIS < targetStateIndex){
+   if(groundStateIndex < targetStateIndex && numberExcitedStatesCIS < targetStateIndex){
       stringstream ss;
       ss << this->errorMessageNonValidExcitedStatesMD;
       ss << this->errorMessageElecState << targetStateIndex << endl;
@@ -1389,10 +1491,9 @@ void InputParser::ValidateRpmdConditions(const Molecule& molecule) const{
    int targetStateIndex = Parameters::GetInstance()->GetElectronicStateIndexRPMD();
    TheoryType theory = Parameters::GetInstance()->GetCurrentTheory();
    // Validate theory
-   if(theory == CNDO2 || theory == INDO || (theory == ZINDOS && groundStateIndex < targetStateIndex)){
+   if(theory == CNDO2 || theory == INDO ){
       stringstream ss;
-      ss << this->errorMessageNonValidExcitedStatesRPMD;
-      ss << this->errorMessageElecState << targetStateIndex << endl;
+      ss << this->errorMessageNonValidTheoriesRPMD;
       ss << this->errorMessageTheory << TheoryTypeStr(theory) << endl;
       throw MolDSException(ss.str());
    }
@@ -1403,7 +1504,7 @@ void InputParser::ValidateRpmdConditions(const Molecule& molecule) const{
       this->ValidateCisConditions(molecule);
    }
    int numberExcitedStatesCIS = Parameters::GetInstance()->GetNumberExcitedStatesCIS();
-   if(numberExcitedStatesCIS < targetStateIndex){
+   if(groundStateIndex < targetStateIndex && numberExcitedStatesCIS < targetStateIndex){
       stringstream ss;
       ss << this->errorMessageNonValidExcitedStatesRPMD;
       ss << this->errorMessageElecState << targetStateIndex << endl;
@@ -1453,9 +1554,9 @@ void InputParser::ValidateOptimizationConditions(const Molecule& molecule) const
    int targetStateIndex = Parameters::GetInstance()->GetElectronicStateIndexOptimization();
    TheoryType theory = Parameters::GetInstance()->GetCurrentTheory();
    // Validate theory
-   if(theory == CNDO2 || theory == INDO || (theory == ZINDOS && groundStateIndex < targetStateIndex)){
+   if(theory == CNDO2 || theory == INDO ){
       stringstream ss;
-      ss << this->errorMessageNonValidExcitedStatesOptimization;
+      ss << this->errorMessageNonValidTheoriesOptimization;
       ss << this->errorMessageElecState << targetStateIndex << endl;
       ss << this->errorMessageTheory << TheoryTypeStr(theory) << endl;
       throw MolDSException(ss.str());
@@ -1467,7 +1568,7 @@ void InputParser::ValidateOptimizationConditions(const Molecule& molecule) const
       this->ValidateCisConditions(molecule);
    }
    int numberExcitedStatesCIS = Parameters::GetInstance()->GetNumberExcitedStatesCIS();
-   if(numberExcitedStatesCIS < targetStateIndex){
+   if(groundStateIndex < targetStateIndex && numberExcitedStatesCIS < targetStateIndex){
       stringstream ss;
       ss << this->errorMessageNonValidExcitedStatesOptimization;
       ss << this->errorMessageElecState << targetStateIndex << endl;
@@ -1501,6 +1602,7 @@ void InputParser::OutputMolecularBasics(Molecule* molecule) const{
    molecule->OutputConfiguration();
    molecule->OutputXyzCOM();
    molecule->OutputXyzCOC();
+   molecule->OutputEpcs();
 }
 
 void InputParser::OutputScfConditions() const{
@@ -1585,6 +1687,15 @@ void InputParser::OutputCisConditions() const{
    }
    this->OutputLog("\n");
 
+   if(Parameters::GetInstance()->RequiresMullikenCIS()){
+      vector<int>* indeces = Parameters::GetInstance()->GetElectronicStateIndecesMullikenCIS();
+      for(int i=0; i<indeces->size(); i++){
+         this->OutputLog(boost::format("%s%d\n") % this->messageCisMulliken.c_str()
+                                                 % (*indeces)[i]);
+      }
+      this->OutputLog("\n");
+   }
+   
    this->OutputLog("\n");
 }
 
@@ -1686,9 +1797,14 @@ void InputParser::OutputOptimizationConditions() const{
          break;
 			case BFGSMethod:
          this->OutputLog(boost::format("%s%lf\n") % this->messageOptimizationInitialTrustRadius.c_str()
-                                                    % Parameters::GetInstance()->GetInitialTrustRadiusOptimization());
+                                                  % Parameters::GetInstance()->GetInitialTrustRadiusOptimization());
          this->OutputLog(boost::format("%s%lf\n") % this->messageOptimizationMaxNormStep.c_str()
-                                                    % Parameters::GetInstance()->GetMaxNormStepOptimization());
+                                                  % Parameters::GetInstance()->GetMaxNormStepOptimization());
+			case GEDIISMethod:
+         this->OutputLog(boost::format("%s%lf\n") % this->messageOptimizationInitialTrustRadius.c_str()
+                                                  % Parameters::GetInstance()->GetInitialTrustRadiusOptimization());
+         this->OutputLog(boost::format("%s%lf\n") % this->messageOptimizationMaxNormStep.c_str()
+                                                  % Parameters::GetInstance()->GetMaxNormStepOptimization());
       default:
          break;
    }
@@ -1710,12 +1826,12 @@ void InputParser::OutputMOPlotConditions() const{
       this->OutputLog(boost::format("%s%d\n") % this->messageMOPlotIndex.c_str() 
                                               % (*moIndeces)[i]);
    }
-   int* gridNum = Parameters::GetInstance()->GetGridNumberMOPlot();
+   const int* gridNum = Parameters::GetInstance()->GetGridNumberMOPlot();
    this->OutputLog(boost::format("%s%d %d %d\n") % this->messageMOPlotGridNumber.c_str() 
                                                  % gridNum[XAxis] 
                                                  % gridNum[YAxis]
                                                  % gridNum[ZAxis]);
-   double* frameLength = Parameters::GetInstance()->GetFrameLengthMOPlot();
+   const double* frameLength = Parameters::GetInstance()->GetFrameLengthMOPlot();
    double ang2AU = Parameters::GetInstance()->GetAngstrom2AU();
    this->OutputLog(boost::format("%s%e %e %e\n") % this->messageMOPlotFrameLength.c_str() 
                                                  % (frameLength[XAxis]/ang2AU) 
@@ -1734,12 +1850,12 @@ void InputParser::OutputHolePlotConditions() const{
       this->OutputLog(boost::format("%s%d\n") % this->messageHolePlotElecIndex.c_str() 
                                               % (*moIndeces)[i]);
    }
-   int* gridNum = Parameters::GetInstance()->GetGridNumberHolePlot();
+   const int* gridNum = Parameters::GetInstance()->GetGridNumberHolePlot();
    this->OutputLog(boost::format("%s%d %d %d\n") % this->messageHolePlotGridNumber.c_str() 
                                                  % gridNum[XAxis] 
                                                  % gridNum[YAxis]
                                                  % gridNum[ZAxis]);
-   double* frameLength = Parameters::GetInstance()->GetFrameLengthHolePlot();
+   const double* frameLength = Parameters::GetInstance()->GetFrameLengthHolePlot();
    double ang2AU = Parameters::GetInstance()->GetAngstrom2AU();
    this->OutputLog(boost::format("%s%e %e %e\n") % this->messageHolePlotFrameLength.c_str() 
                                                  % (frameLength[XAxis]/ang2AU) 
@@ -1753,17 +1869,17 @@ void InputParser::OutputHolePlotConditions() const{
 
 void InputParser::OutputParticlePlotConditions() const{
    this->OutputLog(this->messageParticlePlotConditions);
-   vector<int>* moIndeces = Parameters::GetInstance()->GetElecIndecesParticlePlot();
+   const vector<int>* moIndeces = Parameters::GetInstance()->GetElecIndecesParticlePlot();
    for(int i=0; i<moIndeces->size(); i++){
       this->OutputLog(boost::format("%s%d\n") % this->messageParticlePlotElecIndex.c_str() 
                                               % (*moIndeces)[i]);
    }
-   int* gridNum = Parameters::GetInstance()->GetGridNumberParticlePlot();
+   const int* gridNum = Parameters::GetInstance()->GetGridNumberParticlePlot();
    this->OutputLog(boost::format("%s%d %d %d\n") % this->messageParticlePlotGridNumber.c_str() 
                                                  % gridNum[XAxis] 
                                                  % gridNum[YAxis]
                                                  % gridNum[ZAxis]);
-   double* frameLength = Parameters::GetInstance()->GetFrameLengthParticlePlot();
+   const double* frameLength = Parameters::GetInstance()->GetFrameLengthParticlePlot();
    double ang2AU = Parameters::GetInstance()->GetAngstrom2AU();
    this->OutputLog(boost::format("%s%e %e %e\n") % this->messageParticlePlotFrameLength.c_str() 
                                                  % (frameLength[XAxis]/ang2AU) 
