@@ -1,5 +1,5 @@
 //************************************************************************//
-// Copyright (C) 2011-2012 Mikiya Fujii                                   // 
+// Copyright (C) 2011-2013 Mikiya Fujii                                   //
 //                                                                        // 
 // This file is part of MolDS.                                            // 
 //                                                                        // 
@@ -24,16 +24,20 @@
 #include<string>
 #include<vector>
 #include<stdexcept>
+#include<omp.h>
 #include<boost/format.hpp>
+#include"Enums.h"
+#include"Uncopyable.h"
 #include"PrintController.h"
 #include"MolDSException.h"
-#include"Uncopyable.h"
-#include"../wrappers/Lapack.h"
-#include"Enums.h"
-#include"MathUtilities.h"
 #include"MallocerFreer.h"
+#include"../mpi/MpiInt.h"
+#include"../mpi/MpiProcess.h"
+#include"../wrappers/Lapack.h"
+#include"MathUtilities.h"
 #include"EularAngle.h"
 #include"Parameters.h"
+#include"RealSphericalHarmonicsIndex.h"
 #include"atoms/Atom.h"
 #include"factories/AtomFactory.h"
 #include"Molecule.h"
@@ -51,16 +55,32 @@ Molecule::Molecule(const Molecule& rhs){
 }
 
 Molecule& Molecule::operator=(const Molecule& rhs){
-   double* oldXyzCOM = this->xyzCOM;
-   double* oldXyzCOC = this->xyzCOC;
+   double*  oldXyzCOM = this->xyzCOM;
+   double*  oldXyzCOC = this->xyzCOC;
+   double** oldDistanceAtoms     = this->distanceAtoms;
+   double** oldDistanceEpcs      = this->distanceEpcs;
+   double** oldDistanceAtomsEpcs = this->distanceAtomsEpcs;
    vector<Atom*>* oldAtomVect = this->atomVect;
+   vector<Atom*>* oldEpcVect  = this->epcVect;
    this->CopyInitialize(rhs);
-   this->Finalize(&oldAtomVect, &oldXyzCOM, &oldXyzCOC);
+   this->Finalize(&oldAtomVect, 
+                  &oldEpcVect, 
+                  &oldXyzCOM, 
+                  &oldXyzCOC, 
+                  &oldDistanceAtoms,
+                  &oldDistanceEpcs,
+                  &oldDistanceAtomsEpcs);
    return *this;
 }
 
 Molecule::~Molecule(){
-   this->Finalize(&this->atomVect, &this->xyzCOM, &this->xyzCOC);
+   this->Finalize(&this->atomVect, 
+                  &this->epcVect, 
+                  &this->xyzCOM, 
+                  &this->xyzCOC, 
+                  &this->distanceAtoms,
+                  &this->distanceEpcs,
+                  &this->distanceAtomsEpcs);
    //this->OutputLog("molecule deleted\n");
 }
 
@@ -70,44 +90,111 @@ void Molecule::CopyInitialize(const Molecule& rhs){
       this->xyzCOM[i] = rhs.xyzCOM[i];
       this->xyzCOC[i] = rhs.xyzCOC[i];
    }
-   this->wasCalculatedXyzCOM = rhs.wasCalculatedXyzCOM;
-   this->wasCalculatedXyzCOC = rhs.wasCalculatedXyzCOC;
    this->totalNumberAOs = rhs.totalNumberAOs;
    this->totalNumberValenceElectrons = rhs.totalNumberValenceElectrons;
    this->totalCoreMass = rhs.totalCoreMass;
-   for(int i=0; i<rhs.atomVect->size(); i++){
-      Atom* atom = (*rhs.atomVect)[i];
-      this->atomVect->push_back(AtomFactory::Create(atom->GetAtomType(),
-                                                    atom->GetXyz()[XAxis],
-                                                    atom->GetXyz()[YAxis],
-                                                    atom->GetXyz()[ZAxis],
-                                                    atom->GetPxyz()[XAxis],
-                                                    atom->GetPxyz()[YAxis],
-                                                    atom->GetPxyz()[ZAxis]));
-      (*this->atomVect)[i]->SetFirstAOIndex(atom->GetFirstAOIndex());
-   }                                                                     
+   if(rhs.atomVect != NULL){
+      int atomNum = rhs.atomVect->size();
+      for(int i=0; i<atomNum; i++){
+         Atom* atom = (*rhs.atomVect)[i];
+         this->atomVect->push_back(AtomFactory::Create(atom->GetAtomType(),
+                                                       atom->GetIndex(),
+                                                       atom->GetXyz()[XAxis],
+                                                       atom->GetXyz()[YAxis],
+                                                       atom->GetXyz()[ZAxis],
+                                                       atom->GetPxyz()[XAxis],
+                                                       atom->GetPxyz()[YAxis],
+                                                       atom->GetPxyz()[ZAxis]));
+         (*this->atomVect)[i]->SetFirstAOIndex(atom->GetFirstAOIndex());
+      }
+      MallocerFreer::GetInstance()->Malloc<double>(&this->distanceAtoms, atomNum, atomNum);
+      for(int i=0; i<atomNum; i++){
+         for(int j=0; j<atomNum; j++){
+            this->distanceAtoms[i][j] = rhs.distanceAtoms[i][j];
+         }
+      }
+   }
+   if(rhs.epcVect != NULL){
+      int epcNum = rhs.epcVect->size();
+      for(int i=0; i<epcNum; i++){
+         Atom* epc = (*rhs.epcVect)[i];
+         this->epcVect->push_back(
+            AtomFactory::Create(EPC,
+                                epc->GetIndex(),
+                                epc->GetXyz()[XAxis],
+                                epc->GetXyz()[YAxis],
+                                epc->GetXyz()[ZAxis],
+                                epc->GetPxyz()[XAxis],
+                                epc->GetPxyz()[YAxis],
+                                epc->GetPxyz()[ZAxis],
+                                epc->GetCoreCharge()));
+         (*this->epcVect)[i]->SetFirstAOIndex(epc->GetFirstAOIndex());
+      }
+      MallocerFreer::GetInstance()->Malloc<double>(&this->distanceEpcs, epcNum, epcNum);
+      for(int i=0; i<epcNum; i++){
+         for(int j=0; j<epcNum; j++){
+            this->distanceEpcs[i][j] = rhs.distanceEpcs[i][j];
+         }
+      }
+   }
+   if(rhs.atomVect != NULL && rhs.epcVect != NULL){
+      int atomNum = rhs.atomVect->size();
+      int epcNum = rhs.epcVect->size();
+      MallocerFreer::GetInstance()->Malloc<double>(&this->distanceAtomsEpcs, atomNum, epcNum);
+      for(int i=0; i<atomNum; i++){
+         for(int j=0; j<epcNum; j++){
+            this->distanceAtomsEpcs[i][j] = rhs.distanceAtomsEpcs[i][j];
+         }
+      }
+   }
 }
 
 void Molecule::Initialize(){
    this->SetMessages();
-   this->wasCalculatedXyzCOM = false;
-   this->wasCalculatedXyzCOC = false;
-   this->xyzCOM = NULL;
-   this->xyzCOC = NULL;
+   this->xyzCOM             = NULL;
+   this->xyzCOC             = NULL;
+   this->distanceAtoms      = NULL;
+   this->distanceEpcs       = NULL;
+   this->distanceAtomsEpcs  = NULL;
+   this->atomVect           = NULL;
+   this->epcVect            = NULL;
    try{
       this->atomVect = new vector<Atom*>;
+      this->epcVect  = new vector<Atom*>;
       MallocerFreer::GetInstance()->Malloc<double>(&this->xyzCOM, CartesianType_end);
       MallocerFreer::GetInstance()->Malloc<double>(&this->xyzCOC, CartesianType_end);
    }
    catch(exception ex){
-      this->Finalize(&this->atomVect, &this->xyzCOM, &this->xyzCOC);
+      this->Finalize(&this->atomVect, 
+                     &this->epcVect, 
+                     &this->xyzCOM, 
+                     &this->xyzCOC, 
+                     &this->distanceAtoms,
+                     &this->distanceEpcs,
+                     &this->distanceAtomsEpcs);
       throw MolDSException(ex.what());
    }
 }
 
-void Molecule::Finalize(vector<Atom*>** atomVect, double** xyzCOM, double**xyzCOC){
+void Molecule::Finalize(vector<Atom*>** atomVect, 
+                        vector<Atom*>** epcVect, 
+                        double** xyzCOM, 
+                        double** xyzCOC, 
+                        double*** distanceAtoms,
+                        double*** distanceEpcs,
+                        double*** distanceAtomsEpcs){
+   MallocerFreer::GetInstance()->Free<double>(xyzCOM, CartesianType_end);
+   MallocerFreer::GetInstance()->Free<double>(xyzCOC, CartesianType_end);
+   int atomNum=0;
+   int epcNum = 0;
+   if(*atomVect != NULL && *epcVect != NULL){
+      atomNum = (*atomVect)->size();
+      epcNum = (*epcVect)->size();
+      MallocerFreer::GetInstance()->Free<double>(distanceAtomsEpcs, atomNum, epcNum);
+   }
    if(*atomVect != NULL){
-      for(int i=0; i<(*atomVect)->size(); i++){
+      atomNum = (*atomVect)->size();
+      for(int i=0; i<atomNum; i++){
          if((**atomVect)[i] != NULL){
             delete (**atomVect)[i];
             (**atomVect)[i] = NULL;
@@ -117,15 +204,35 @@ void Molecule::Finalize(vector<Atom*>** atomVect, double** xyzCOM, double**xyzCO
       delete *atomVect;
       *atomVect = NULL;
       //this->OutputLog("atomVect deleted\n");
+      MallocerFreer::GetInstance()->Free<double>(distanceAtoms, atomNum, atomNum);
+   }
+   if(*epcVect != NULL){
+      epcNum = (*epcVect)->size();
+      for(int i=0; i<epcNum; i++){
+         if((**epcVect)[i] != NULL){
+            delete (**epcVect)[i];
+            (**epcVect)[i] = NULL;
+         }
+      }
+      (*epcVect)->clear();
+      delete *epcVect;
+      *epcVect = NULL;
+      //this->OutputLog("epcVect deleted\n");
+      MallocerFreer::GetInstance()->Free<double>(distanceEpcs, epcNum, epcNum);
    }
-   MallocerFreer::GetInstance()->Free<double>(xyzCOM, CartesianType_end);
-   MallocerFreer::GetInstance()->Free<double>(xyzCOC, CartesianType_end);
 }
 
 void Molecule::SetMessages(){
-   this->errorMessageGetAtomVectNull = "Error in base::Molecule::GetAtomVect: atomVect is NULL.\n";
-   this->errorMessageGetXyzCOCNull = "Error in base::Molecule::GetXyzCOC: xyzCOC is NULL.\n";
+   this->errorMessageGetAtomNull = "Error in base::Molecule::GetAtom: atomVect is NULL.\n";
+   this->errorMessageGetEPCNull  = "Error in base::Molecule::GetEnviromentalPointCharge: enviromentalPointChargeVect is NULL.\n";
+   this->errorMessageAddAtomNull = "Error in base::Molecule::AddAtom: atomVect is NULL.\n";
+   this->errorMessageAddEPCNull  = "Error in base::Molecule::AddEnviromentalPointCharge: enviromentalPointChargeVect is NULL.\n";
+   this->errorMessageGetNumberAtomsNull = "Error in base::Molecule::GetNumberAtoms: atomVect is NULL.\n";
+   this->errorMessageGetNumberEPCsNull  = "Error in base::Molecule::GetNumberEnviromentalPointChargess: epcVect is NULL.\n";
    this->errorMessageGetXyzCOMNull = "Error in base::Molecule::GetXyzCOM: xyzCOM is NULL.\n";
+   this->errorMessageGetXyzCOCNull = "Error in base::Molecule::GetXyzCOC: xyzCOC is NULL.\n";
+   this->errorMessageCalcXyzCOMNull = "Error in base::Molecule::CalcXyzCOM: xyzCOM is NULL.\n";
+   this->errorMessageCalcXyzCOCNull = "Error in base::Molecule::CalcXyzCOC: xyzCOC is NULL.\n";
    this->messageTotalNumberAOs = "\tTotal number of valence AOs: ";
    this->messageTotalNumberAtoms = "\tTotal number of atoms: ";
    this->messageTotalNumberValenceElectrons = "\tTotal number of valence electrons: ";
@@ -133,6 +240,9 @@ void Molecule::SetMessages(){
    this->messageAtomCoordinates = "\tAtom coordinates:";
    this->messageAtomMomenta = "\tAtom momenta:";
    this->messageAtomMomentaTitle = "\t\t\t| i-th | atom type |   px[a.u.]   |   py[a.u.]   |   pz[a.u.]   |\t\t|   px[u]   |   py[u]   |   pz[u]   | [u] = [(g/Mol)*(angst/fs)]\n";
+   this->messageEpcConfiguration = "\tEnvironmental Point Charge(EPC) configuration:\n";
+   this->messageEpcCoordinates = "\tEPC coordinates:";
+   this->messageEpcCoordinatesTitle = "\t\t\t\t| i-th |  charge[a.u.]  |   x[a.u.]   |   y[a.u.]   |   z[a.u.]   |\t\t|  x[angst.]  |  y[angst.]  |  z[angst.]  |\n";
    this->messageCOM = "\tCenter of Mass:";
    this->messageCOC = "\tCenter of Core:";
    this->messageCOMTitle = "\t\t\t|   x[a.u.]   |   y[a.u.]   |   z[a.u.]   |\t\t|  x[angst.]  |  y[angst.]  |  z[angst.]  |\n";
@@ -160,46 +270,40 @@ void Molecule::SetMessages(){
 }
 
 void Molecule::AddAtom(Atom* atom){
+#ifdef MOLDS_DBG
+   if(this->atomVect==NULL) throw MolDSException(this->errorMessageAddAtomNull);
+#endif
    this->atomVect->push_back(atom);
 }
 
-double* Molecule::GetXyzCOM() const{
-   if(this->xyzCOM==NULL){
-      stringstream ss;
-      ss << this->errorMessageGetXyzCOMNull;
-      throw MolDSException(ss.str());
-   }
+void Molecule::AddEpc(Atom* epc){
+#ifdef MOLDS_DBG
+   if(this->epcVect==NULL) throw MolDSException(this->errorMessageAddEPCNull);
+#endif
+   this->epcVect->push_back(epc);
+}
+
+double const* Molecule::GetXyzCOM() const{
+#ifdef MOLDS_DBG
+   if(this->xyzCOM==NULL) throw MolDSException(this->errorMessageGetXyzCOMNull);
+#endif
    return this->xyzCOM;
 }
 
-double* Molecule::GetXyzCOM(){
-   if(!this->wasCalculatedXyzCOM){
-      this->CalcXyzCOM();
-   }
-   return this->xyzCOM;
-}
-
-double* Molecule::GetXyzCOC() const{
-   if(this->xyzCOC==NULL){
-      stringstream ss;
-      ss << this->errorMessageGetXyzCOCNull;
-      throw MolDSException(ss.str());
-   }
-   return this->xyzCOC;
-}
-
-double* Molecule::GetXyzCOC(){
-   if(!this->wasCalculatedXyzCOC){
-      this->CalcXyzCOC();
-   }
+double const* Molecule::GetXyzCOC() const{
+#ifdef MOLDS_DBG
+   if(this->xyzCOC==NULL) throw MolDSException(this->errorMessageGetXyzCOCNull);
+#endif
    return this->xyzCOC;
 }
 
 void Molecule::CalcXyzCOM(){
-   MallocerFreer::GetInstance()->Malloc<double>(&this->xyzCOM, CartesianType_end);
-   double totalAtomicMass = 0.0;
+#ifdef MOLDS_DBG
+   if(this->xyzCOM==NULL) throw MolDSException(this->errorMessageCalcXyzCOMNull);
+#endif
+   double  totalAtomicMass = 0.0;
    double* atomicXyz;
-   double atomicMass = 0.0;
+   double  atomicMass = 0.0;
 
    for(int j=0; j<3; j++){
       this->xyzCOM[j] = 0.0;
@@ -217,14 +321,15 @@ void Molecule::CalcXyzCOM(){
    for(int i=0; i<3; i++){
       this->xyzCOM[i]/=totalAtomicMass;
    }
-   this->wasCalculatedXyzCOM = true;
 }
 
 void Molecule::CalcXyzCOC(){
-   MallocerFreer::GetInstance()->Malloc<double>(&this->xyzCOC, CartesianType_end);
-   double totalCoreMass = 0.0;
+#ifdef MOLDS_DBG
+   if(this->xyzCOC==NULL) throw MolDSException(this->errorMessageCalcXyzCOCNull);
+#endif
+   double  totalCoreMass = 0.0;
    double* atomicXyz;
-   double coreMass = 0.0;
+   double  coreMass = 0.0;
 
    for(int j=0; j<3; j++){
       this->xyzCOC[j] = 0.0;
@@ -242,19 +347,76 @@ void Molecule::CalcXyzCOC(){
    for(int i=0; i<3; i++){
       this->xyzCOC[i]/=totalCoreMass;
    }
-   this->wasCalculatedXyzCOC = true;
 }
 
-int Molecule::GetTotalNumberAOs() const{
-   return this->totalNumberAOs;
+void Molecule::CalcDistanceAtoms(){
+   if(this->distanceAtoms==NULL){
+      MallocerFreer::GetInstance()->Malloc<double>(&this->distanceAtoms, this->atomVect->size(), this->atomVect->size());
+   }
+   for(int a=0; a<this->atomVect->size(); a++){
+      const Atom& atomA = *(*this->atomVect)[a];
+      for(int b=a; b<this->atomVect->size(); b++){
+         const Atom& atomB = *(*this->atomVect)[b];
+         double distance=0.0;
+         distance = sqrt( pow(atomA.GetXyz()[0] - atomB.GetXyz()[0], 2.0)
+                         +pow(atomA.GetXyz()[1] - atomB.GetXyz()[1], 2.0)
+                         +pow(atomA.GetXyz()[2] - atomB.GetXyz()[2], 2.0) );
+         this->distanceAtoms[a][b] = distance;
+         this->distanceAtoms[b][a] = distance;
+      }
+   }
+}
+
+void Molecule::CalcDistanceEpcs(){
+   if(this->epcVect == NULL){return;}
+   if(this->distanceEpcs==NULL){
+      MallocerFreer::GetInstance()->Malloc<double>(&this->distanceEpcs, this->epcVect->size(), this->epcVect->size());
+   }
+   for(int a=0; a<this->epcVect->size(); a++){
+      const Atom& epcA = *(*this->epcVect)[a];
+      for(int b=a; b<this->epcVect->size(); b++){
+         const Atom& epcB = *(*this->epcVect)[b];
+         double distance=0.0;
+         distance = sqrt( pow(epcA.GetXyz()[0] - epcB.GetXyz()[0], 2.0)
+                         +pow(epcA.GetXyz()[1] - epcB.GetXyz()[1], 2.0)
+                         +pow(epcA.GetXyz()[2] - epcB.GetXyz()[2], 2.0) );
+         this->distanceEpcs[a][b] = distance;
+         this->distanceEpcs[b][a] = distance;
+      }
+   }
+}
+
+void Molecule::CalcDistanceAtomsEpcs(){
+   if(this->epcVect == NULL){return;}
+   if(this->distanceAtomsEpcs==NULL){
+      MallocerFreer::GetInstance()->Malloc<double>(&this->distanceAtomsEpcs, this->atomVect->size(), this->epcVect->size());
+   }
+   for(int a=0; a<this->atomVect->size(); a++){
+      const Atom& atom = *(*this->atomVect)[a];
+      for(int b=0; b<this->epcVect->size(); b++){
+         const Atom& epc = *(*this->epcVect)[b];
+         double distance=0.0;
+         distance = sqrt( pow(atom.GetXyz()[0] - epc.GetXyz()[0], 2.0)
+                         +pow(atom.GetXyz()[1] - epc.GetXyz()[1], 2.0)
+                         +pow(atom.GetXyz()[2] - epc.GetXyz()[2], 2.0) );
+         this->distanceAtomsEpcs[a][b] = distance;
+      }
+   }
 }
 
 void Molecule::CalcBasics(){
    this->CalcTotalNumberAOs();
    this->CalcTotalNumberValenceElectrons();
    this->CalcTotalCoreMass();
+   this->CalcBasicsConfiguration();
+}
+
+void Molecule::CalcBasicsConfiguration(){
    this->CalcXyzCOM();
    this->CalcXyzCOC();
+   this->CalcDistanceAtoms();
+   this->CalcDistanceEpcs();
+   this->CalcDistanceAtomsEpcs();
 }
 
 void Molecule::CalcTotalNumberAOs(){
@@ -265,10 +427,6 @@ void Molecule::CalcTotalNumberAOs(){
    }
 }
 
-int Molecule::GetTotalNumberValenceElectrons() const{
-   return this->totalNumberValenceElectrons;
-}
-
 void Molecule::CalcTotalNumberValenceElectrons(){
    this->totalNumberValenceElectrons = 0; 
    for(int i=0; i<this->atomVect->size(); i++){
@@ -276,10 +434,6 @@ void Molecule::CalcTotalNumberValenceElectrons(){
    }
 }
 
-double Molecule::GetTotalCoreMass() const{
-   return this->totalCoreMass;
-}
-
 void Molecule::CalcTotalCoreMass(){
    this->totalCoreMass = 0; 
    for(int i=0; i<this->atomVect->size(); i++){
@@ -330,6 +484,27 @@ void Molecule::OutputMomenta() const{
    this->OutputLog("\n");
 }
 
+void Molecule::OutputEpcs() const{
+   if(this->epcVect == NULL || this->epcVect->size() <= 0) {return;}
+   double ang2AU = Parameters::GetInstance()->GetAngstrom2AU();
+   this->OutputLog(this->messageEpcConfiguration);
+   this->OutputLog(this->messageEpcCoordinatesTitle);
+   for(int a=0; a<this->epcVect->size(); a++){
+      const Atom& atom = *(*this->epcVect)[a];
+      this->OutputLog(boost::format("%s\t%d\t%e\t%e\t%e\t%e\t\t%e\t%e\t%e\n") 
+         % this->messageEpcCoordinates
+         % a
+         % atom.GetCoreCharge()
+         % atom.GetXyz()[0]
+         % atom.GetXyz()[1]
+         % atom.GetXyz()[2]
+         % (atom.GetXyz()[0]/ang2AU)
+         % (atom.GetXyz()[1]/ang2AU)
+         % (atom.GetXyz()[2]/ang2AU));
+   }
+   this->OutputLog("\n");
+}
+
 void Molecule::OutputXyzCOM() const{
    double ang2AU = Parameters::GetInstance()->GetAngstrom2AU();
    this->OutputLog(this->messageCOMTitle);
@@ -405,9 +580,7 @@ void Molecule::OutputInertiaTensorOrigin(double* inertiaTensorOrigin) const{
 
 void Molecule::CalcPrincipalAxes(){
    this->OutputLog(this->messageStartPrincipalAxes);
-   if(!this->wasCalculatedXyzCOM){
-      this->CalcXyzCOM();
-   }
+   this->CalcXyzCOM();
    double inertiaTensorOrigin[3] = {this->xyzCOM[0], this->xyzCOM[1], this->xyzCOM[2]};
    if(Parameters::GetInstance()->GetInertiaTensorOrigin() != NULL){
       inertiaTensorOrigin[0] = Parameters::GetInstance()->GetInertiaTensorOrigin()[0];
@@ -476,9 +649,7 @@ void Molecule::Rotate(){
    this->OutputLog(this->messageStartRotate);
 
    // Default values are set if some conditions are not specified.
-   if(!this->wasCalculatedXyzCOM){
-      this->CalcXyzCOM();
-   }
+   this->CalcXyzCOM();
    double rotatingOrigin[3] = {this->xyzCOM[0], this->xyzCOM[1], this->xyzCOM[2]};
    if(Parameters::GetInstance()->GetRotatingOrigin() != NULL){
       rotatingOrigin[0] = Parameters::GetInstance()->GetRotatingOrigin()[0];
@@ -486,10 +657,10 @@ void Molecule::Rotate(){
       rotatingOrigin[2] = Parameters::GetInstance()->GetRotatingOrigin()[2];
    }
 
-   RotatingType rotatingType = Parameters::GetInstance()->GetRotatingType();
-   double* rotatingAxis = Parameters::GetInstance()->GetRotatingAxis();
-   EularAngle rotatingEularAngles = Parameters::GetInstance()->GetRotatingEularAngles();
-   double rotatingAngle = Parameters::GetInstance()->GetRotatingAngle();
+   RotatingType  rotatingType        = Parameters::GetInstance()->GetRotatingType();
+   const double* rotatingAxis        = Parameters::GetInstance()->GetRotatingAxis();
+   EularAngle    rotatingEularAngles = Parameters::GetInstance()->GetRotatingEularAngles();
+   double        rotatingAngle       = Parameters::GetInstance()->GetRotatingAngle();
 
    this->OutputRotatingConditions(rotatingType, rotatingOrigin, 
                                   rotatingAxis, rotatingAngle, 
@@ -639,12 +810,8 @@ void Molecule::Translate(){
          atom.GetXyz()[1] += y;
          atom.GetXyz()[2] += z;
    }
-   
-   this->wasCalculatedXyzCOM = false;
    this->CalcXyzCOM();
-   this->wasCalculatedXyzCOC = false;
    this->CalcXyzCOC();
-
    this->OutputConfiguration();
    this->OutputXyzCOM();
    this->OutputXyzCOC();
@@ -664,22 +831,6 @@ void Molecule::OutputTranslatingConditions(double const* translatingDifference)
       % (translatingDifference[2]/angst2AU));
 }
 
-double Molecule::GetDistanceAtoms(int indexAtomA, int indexAtomB) const{
-   const Atom& atomA = *(*this->atomVect)[indexAtomA];
-   const Atom& atomB = *(*this->atomVect)[indexAtomB];
-   return this->GetDistanceAtoms(atomA, atomB);
-}
-
-double Molecule::GetDistanceAtoms(const Atom& atomA, const Atom& atomB) const{
-
-   double distance=0.0;
-   distance = sqrt( pow(atomA.GetXyz()[0] - atomB.GetXyz()[0], 2.0)
-                   +pow(atomA.GetXyz()[1] - atomB.GetXyz()[1], 2.0)
-                   +pow(atomA.GetXyz()[2] - atomB.GetXyz()[2], 2.0) );
-   return distance;
-
-}
-
 void Molecule::SynchronizeConfigurationTo(const Molecule& ref){
    for(int a=0; a<this->GetNumberAtoms(); a++){
       Atom& atom = *this->GetAtom(a);
@@ -688,8 +839,7 @@ void Molecule::SynchronizeConfigurationTo(const Molecule& ref){
          atom.GetXyz()[i] = refAtom.GetXyz()[i];
       }
    }
-   this->CalcXyzCOC();
-   this->CalcXyzCOM();
+   this->CalcBasicsConfiguration();
 }
 
 void Molecule::SynchronizeMomentaTo(const Molecule& ref){
@@ -711,8 +861,89 @@ void Molecule::SynchronizePhaseSpacePointTo(const Molecule& ref){
          atom.GetPxyz()[i] = refAtom.GetPxyz()[i];
       }
    }
-   this->CalcXyzCOC();
-   this->CalcXyzCOM();
+   this->CalcBasicsConfiguration();
+}
+
+void Molecule::BroadcastConfigurationToAllProcesses(int root) const{
+   int numTransported = this->GetNumberAtoms()*CartesianType_end;
+   double* tmp=NULL;
+   try{
+      MolDS_base::MallocerFreer::GetInstance()->Malloc<double>(&tmp, numTransported);
+      for(int a=0; a<this->GetNumberAtoms(); a++){
+         Atom& atom = *this->GetAtom(a);
+         for(int i=0; i<CartesianType_end; i++){
+            tmp[a*CartesianType_end+i] = atom.GetXyz()[i];
+         }
+      }
+      MolDS_mpi::MpiProcess::GetInstance()->Broadcast(tmp, numTransported, root);
+      for(int a=0; a<this->GetNumberAtoms(); a++){
+         Atom& atom = *this->GetAtom(a);
+         for(int i=0; i<CartesianType_end; i++){
+            atom.GetXyz()[i] = tmp[a*CartesianType_end+i];
+         }
+      }
+   }
+   catch(MolDS_base::MolDSException ex){
+      MolDS_base::MallocerFreer::GetInstance()->Free<double>(&tmp, numTransported);
+      throw ex;
+   }
+   MolDS_base::MallocerFreer::GetInstance()->Free<double>(&tmp, numTransported);
+}
+
+void Molecule::BroadcastMomentaToAllProcesses(int root) const{
+   int numTransported = this->GetNumberAtoms()*CartesianType_end;
+   double* tmp=NULL;
+   try{
+      MolDS_base::MallocerFreer::GetInstance()->Malloc<double>(&tmp, numTransported);
+      for(int a=0; a<this->GetNumberAtoms(); a++){
+         Atom& atom = *this->GetAtom(a);
+         for(int i=0; i<CartesianType_end; i++){
+            tmp[a*CartesianType_end+i] = atom.GetPxyz()[i];
+         }
+      }
+      MolDS_mpi::MpiProcess::GetInstance()->Broadcast(tmp, numTransported, root);
+      for(int a=0; a<this->GetNumberAtoms(); a++){
+         Atom& atom = *this->GetAtom(a);
+         for(int i=0; i<CartesianType_end; i++){
+            atom.GetPxyz()[i] = tmp[a*CartesianType_end+i];
+         }
+      }
+   }
+   catch(MolDS_base::MolDSException ex){
+      MolDS_base::MallocerFreer::GetInstance()->Free<double>(&tmp, numTransported);
+      throw ex;
+   }
+   MolDS_base::MallocerFreer::GetInstance()->Free<double>(&tmp, numTransported);
+}
+
+void Molecule::BroadcastPhaseSpacePointToAllProcesses(int root) const{
+   int numTransported = 2*this->GetNumberAtoms()*CartesianType_end;
+   double* tmp=NULL;
+   try{
+      MolDS_base::MallocerFreer::GetInstance()->Malloc<double>(&tmp, numTransported);
+      for(int a=0; a<this->GetNumberAtoms(); a++){
+         Atom& atom = *this->GetAtom(a);
+         for(int i=0; i<CartesianType_end; i++){
+            int k = a*CartesianType_end+i;
+            tmp[2*k  ] = atom.GetXyz() [i];
+            tmp[2*k+1] = atom.GetPxyz()[i];
+         }
+      }
+      MolDS_mpi::MpiProcess::GetInstance()->Broadcast(tmp, numTransported, root);
+      for(int a=0; a<this->GetNumberAtoms(); a++){
+         Atom& atom = *this->GetAtom(a);
+         for(int i=0; i<CartesianType_end; i++){
+            int k = a*CartesianType_end+i;
+            atom.GetXyz() [i] = tmp[2*k  ];
+            atom.GetPxyz()[i] = tmp[2*k+1];
+         }
+      }
+   }
+   catch(MolDS_base::MolDSException ex){
+      MolDS_base::MallocerFreer::GetInstance()->Free<double>(&tmp, numTransported);
+      throw ex;
+   }
+   MolDS_base::MallocerFreer::GetInstance()->Free<double>(&tmp, numTransported);
 }
 
 }
