@@ -1,5 +1,5 @@
 //************************************************************************//
-// Copyright (C) 2011-2012 Mikiya Fujii                                   // 
+// Copyright (C) 2011-2013 Mikiya Fujii                                   //
 //                                                                        // 
 // This file is part of MolDS.                                            // 
 //                                                                        // 
@@ -26,19 +26,23 @@
 #include<vector>
 #include<stdexcept>
 #include<boost/format.hpp>
+#include"../Enums.h"
+#include"../Uncopyable.h"
 #include"../PrintController.h"
 #include"../MolDSException.h"
-#include"../Uncopyable.h"
-#include"../Enums.h"
+#include"../MallocerFreer.h"
+#include"../../mpi/MpiInt.h"
+#include"../../mpi/MpiProcess.h"
 #include"../MathUtilities.h"
 #include"../MallocerFreer.h"
 #include"../EularAngle.h"
+#include"../RealSphericalHarmonicsIndex.h"
 #include"Atom.h"
 using namespace std;
 using namespace MolDS_base;
 
 namespace MolDS_base_atoms{
-Atom::Atom(){
+Atom::Atom(int index){
    this->SetMessages();
    this->xyz = NULL;
    this->pxyz = NULL;
@@ -51,11 +55,17 @@ Atom::Atom(){
       MallocerFreer::GetInstance()->Free<double>(&this->pxyz, CartesianType_end);
       throw MolDSException(ex.what());
    }
+   this->index = index;
 }
 
 Atom::~Atom(){
    MallocerFreer::GetInstance()->Free<double>(&this->xyz, CartesianType_end);
    MallocerFreer::GetInstance()->Free<double>(&this->pxyz, CartesianType_end);
+   int valenceSize = this->valence.size();
+   for(int i=0; i<valenceSize; i++){
+      delete this->realSphericalHarmonicsIndeces[i];
+   }
+   this->realSphericalHarmonicsIndeces.clear();
    //this->OutputLog("atom deleted\n");
 }
 
@@ -130,65 +140,37 @@ void Atom::SetMessages(){
       = "Error in base_atoms::Atom::GetPm3PddgParameterDa: Bad index for parameter Da(daIndex). Only 0, and 1 are permitted.\n";
    this->errorMessageDaIndex  = "daIndex = ";
    this->errorMessageGetXyzCoordinatesNull = "Error in base_atoms::Atom::GetXyz: xyz is NULL\n";
+   this->errorMessageSetXyzCoordinatesNull = "Error in base_atoms::Atom::SetXyz: xyz is NULL\n";
    this->errorMessageGetPxyzMomentaNull  = "Error in base_atoms::Atom::GetPxyz: pxyz is NULL\n";
-}
-
-AtomType Atom::GetAtomType() const{
-   return this->atomType;
-}
-
-double Atom::GetAtomicMass() const{
-   return this->atomicMass;
-}
-
-double Atom::GetCoreMass() const{
-   return this->atomicMass - static_cast<double>(this->numberValenceElectrons);
+   this->errorMessageSetPxyzMomentaNull  = "Error in base_atoms::Atom::SetPxyz: pxyz is NULL\n";
 }
 
 double* Atom::GetXyz() const{
-   if(this->xyz==NULL){
-      stringstream ss;
-      ss << this->errorMessageGetXyzCoordinatesNull;
-      throw MolDSException(ss.str());
-   }
+#ifdef MOLDS_DBG
+   if(this->xyz==NULL) throw MolDSException(this->errorMessageGetXyzCoordinatesNull);
+#endif
    return this->xyz;
 }
 
 double* Atom::GetPxyz() const{
-   if(this->pxyz==NULL){
-      stringstream ss;
-      ss << this->errorMessageGetPxyzMomentaNull;
-      throw MolDSException(ss.str());
-   }
+#ifdef MOLDS_DBG
+   if(this->pxyz==NULL) throw MolDSException(this->errorMessageGetPxyzMomentaNull);
+#endif
    return this->pxyz;
 }
 
 void Atom::SetXyz(double x, double y, double z) const{
-   xyz[0]= x;
-   xyz[1]= y;
-   xyz[2]= z;
+#ifdef MOLDS_DBG
+   if(this->xyz==NULL) throw MolDSException(this->errorMessageSetXyzCoordinatesNull);
+#endif
+   xyz[0]= x; xyz[1]= y; xyz[2]= z;
 }
 
 void Atom::SetPxyz(double px, double py, double pz) const{
-   pxyz[0]= px;
-   pxyz[1]= py;
-   pxyz[2]= pz;
-}
-
-int Atom::GetValenceSize() const{
-   return this->valence.size();
-}
-
-OrbitalType Atom::GetValence(int index) const{
-   return this->valence[index];
-}
-
-double Atom::GetVdWCoefficient() const{
-   return this->vdWCoefficient;
-}
-
-double Atom::GetVdWRadii() const{
-   return this->vdWRadii;
+#ifdef MOLDS_DBG
+   if(this->pxyz==NULL) throw MolDSException(this->errorMessageSetPxyzMomentaNull);
+#endif
+   pxyz[0]= px; pxyz[1]= py; pxyz[2]= pz;
 }
 
 double Atom::GetAtomicBasisValue(double x, 
@@ -353,30 +335,6 @@ double Atom::GetBondingParameter(TheoryType theory, OrbitalType orbital) const{
 
 }
 
-double Atom::GetBondingParameter() const{
-   return this->GetBondingParameter(CNDO2, s);
-}
-
-double Atom::GetCoreCharge() const{
-   return this->coreCharge;
-}
-
-int Atom::GetFirstAOIndex() const{
-   return this->firstAOIndex;
-}
-
-void Atom::SetFirstAOIndex(int firstAOIndex){
-   this->firstAOIndex = firstAOIndex;
-}
-
-int Atom::GetLastAOIndex() const{
-   return this->firstAOIndex + this->valence.size()-1;
-}
-
-ShellType Atom::GetValenceShellType() const{
-   return this->valenceShellType;
-}
-
 int Atom::GetEffectivePrincipalQuantumNumber(ShellType shellType) const{
    if(shellType == k){
       return 1.0;
@@ -396,10 +354,6 @@ int Atom::GetEffectivePrincipalQuantumNumber(ShellType shellType) const{
    }
 }   
 
-int Atom::GetNumberValenceElectrons() const{
-   return this->numberValenceElectrons;
-}
-
 // (1.73) in J. A. Pople book
 double Atom::GetOrbitalExponent(ShellType shellType, 
                                 OrbitalType orbitalType, 
@@ -521,39 +475,6 @@ double Atom::GetOrbitalExponent(ShellType shellType,
 }
 
 
-// Part of Eq. (13) in [BZ_1979]
-double Atom::GetZindoJss() const{
-   return this->zindoF0ss;
-}
-
-// Part of Eq. (13) in [BZ_1979]
-double Atom::GetZindoJsp() const{
-   // F0ss = F0sp
-   return this->zindoF0ss - this->zindoG1sp/6.0;
-}
-
-// Part of Eq. (13) in [BZ_1979]
-double Atom::GetZindoJsd() const{
-   return this->zindoF0sd - this->zindoG2sd/10.0;
-}
-
-// Part of Eq. (13) in [BZ_1979]
-double Atom::GetZindoJpp() const{
-   // F0pp = F0ss
-   return this->zindoF0ss - 2.0*this->zindoF2pp/25.0;
-}
-
-// Part of Eq. (13) in [BZ_1979]
-double Atom::GetZindoJpd() const{
-   // F0pd = F0sd
-   return this->zindoF0sd - this->zindoG1pd/15.0 - 3.0*this->zindoG3pd/70.0;
-}
-
-// Part of Eq. (13) in [BZ_1979]
-double Atom::GetZindoJdd() const{
-   return this->zindoF0dd - 2.0*(this->zindoF2dd + this->zindoF4dd)/63.0;
-}
-
 // (3.72) in J. A. Pople book.
 double Atom::GetCndo2CoreIntegral(OrbitalType orbital, double gamma, bool isGuess) const{
    double value = 0.0;
@@ -766,14 +687,6 @@ double Atom::GetPm3PddgCoreIntegral(OrbitalType orbital) const{
    return value;
 }
 
-double Atom::GetIndoF2() const{
-   return this->indoF2;
-}
-
-double Atom::GetIndoG1() const{
-   return this->indoG1;
-}
-
 double Atom::GetNddoAlpha(TheoryType theory) const{
    double value = 0.0;
    if(theory == MNDO){
@@ -1138,116 +1051,6 @@ double Atom::GetNddoHpp(TheoryType theory) const{
    }
 }
 
-// Table 1 in ref. [RZ_1976], Table 1 in [AEZ_1986], or Table 1 in [GD_1972]
-double Atom::GetZindoF0ss() const{
-   return this->zindoF0ss;
-}
-
-// Table 1 in [AEZ_1986]
-double Atom::GetZindoF0sd() const{
-   return this->zindoF0sd;
-}
-
-// Table 1 in [AEZ_1986]
-double Atom::GetZindoF0dd() const{
-   return this->zindoF0dd;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoG1sp() const{
-   return this->zindoG1sp;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoF2pp() const{
-   return this->zindoF2pp;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoG2sd() const{
-   return this->zindoG2sd;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoG1pd() const{
-   return this->zindoG1pd;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoF2pd() const{
-   return this->zindoF2pd;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoG3pd() const{
-   return this->zindoG3pd;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoF2dd() const{
-   return this->zindoF2dd;
-}
-
-// Table 3 in ref. [BZ_1979]
-double Atom::GetZindoF4dd() const{
-   return this->zindoF4dd;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoF0ssLower() const{
-   return this->zindoF0ss;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoF0sdLower() const{
-   return this->zindoF0sd;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoF0ddLower() const{
-   return this->zindoF0dd;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoG1spLower() const{
-   return this->zindoG1sp/3.0;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoF2ppLower() const{
-   return this->zindoF2pp/25.0;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoG2sdLower() const{
-   return this->zindoG2sd/5.0;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoG1pdLower() const{
-   return this->zindoG1pd/15.0;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoF2pdLower() const{
-   return this->zindoF2pd/35.0;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoG3pdLower() const{
-   return this->zindoG3pd/245.0;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoF2ddLower() const{
-   return this->zindoF2dd/49.0;
-}
-
-// Apendix in ref. [BZ_1979]
-double Atom::GetZindoF4ddLower() const{
-   return this->zindoF4dd/441.0;
-}
-
 double Atom::GetCoreIntegral(OrbitalType orbital, 
                              double gamma, 
                              bool isGuess, 
