@@ -1,5 +1,6 @@
 //************************************************************************//
-// Copyright (C) 2011-2012 Mikiya Fujii                                   // 
+// Copyright (C) 2011-2013 Mikiya Fujii                                   //
+// Copyright (C) 2012-2013 Michihiro Okuyama
 //                                                                        // 
 // This file is part of MolDS.                                            // 
 //                                                                        // 
@@ -21,7 +22,7 @@
 namespace MolDS_cndo{
 
 /***
- *  Refferences for Cndo2 are [PB_1970], [PSS_1965], and [PS_1965].
+ *  References for Cndo2 are [PB_1970], [PSS_1965], and [PS_1965].
  */
 class Cndo2 : public MolDS_base::ElectronicStructure{
 public:
@@ -30,14 +31,14 @@ public:
    virtual void SetMolecule(MolDS_base::Molecule* molecule);
    void DoSCF(bool requiresGuess=true);
    virtual void OutputSCFResults() const;
-   double const* const* GetFockMatrix() const;
-   double const*        GetEnergiesMO() const;
+   double const* const* GetFockMatrix() const{return this->fockMatrix;};
+   double const*        GetEnergiesMO() const{return this->energiesMO;};
    virtual void DoCIS();
    virtual void OutputCISResults() const;
-   double const* const* GetMatrixCIS() const;
-   double const*        GetExcitedEnergies() const;
-   double** GetForce(int elecState);
-   double*** GetForce(const std::vector<int>& elecStates);
+   double const* const* GetMatrixCIS() const{return this->matrixCIS;};
+   double const*        GetExcitedEnergies() const{return this->excitedEnergies;};
+   double const* const*        GetForce(int elecState);
+   double const* const* const* GetForce(const std::vector<int>& elecStates);
    double GetElectronicEnergy(int elecState) const;
    double GetCoreRepulsionEnergy() const;
    double GetVdWCorrectionEnergy() const;
@@ -75,7 +76,7 @@ protected:
    std::string errorMessageGetElectronicEnergyEnergyNotCalculated;
    std::string errorMessageGetElectronicEnergyNumberCISStates;
    std::string errorMessageGetElectronicEnergySetElecState;
-   std::string errorMessageGetElectronicTransitionDipoleMomentBadState;
+   std::string errorMessageCalcElectronicTransitionDipoleMomentBadState;
    std::string errorMessageCalcFrequenciesNormalModesBadTheory;
    std::string errorMessageFromState;
    std::string errorMessageToState;
@@ -84,19 +85,28 @@ protected:
    std::string messageStartSCF;
    std::string messageDoneSCF;
    std::string messageOmpElapsedTimeSCF;
+   std::string messageMullikenAtoms;
+   std::string messageMullikenAtomsTitle;
+   std::string messageUnpairedAtoms;
+   std::string messageUnpairedAtomsTitle;
    std::string messageUnitSec; 
    std::vector<MolDS_base::AtomType> enableAtomTypes;
    MolDS_base::Molecule* molecule;
    MolDS_base::TheoryType theory;
    double       coreRepulsionEnergy;
+   double       coreEpcCoulombEnergy;
    double       vdWCorrectionEnergy;
    int          matrixCISdimension;
    double**     fockMatrix;
    double*      energiesMO;
    double**     orbitalElectronPopulation; //P_{\mu\nu} of (2.50) in J. A. Pople book.
+   double***    orbitalElectronPopulationCIS; 
    double*      atomicElectronPopulation; //P_{AB} of (3.21) in J. A. Pople book.
+   double**     atomicElectronPopulationCIS; 
+   double**     atomicUnpairedPopulationCIS; 
    double**     overlapAOs; // overlap integral between AOs
-   double****** twoElecTwoCore;
+   double****** twoElecsTwoAtomCores;
+   double****** twoElecsAtomEpcCores;
    double***    cartesianMatrix; // cartesian matrix represented by AOs
    double***    electronicTransitionDipoleMoments; // Diagnonal terms are electronic dipole moments of each eigenstates (i.e. electronicDipole[0][0][XAxis] is the x-component of the electronic dipole moment of the ground state. electronicDipole[10][10][XAxis] is the x-component of the electronic dipole moment of the 10-th excited state). Off-diagonal terms are transition dipole moments between eigenstates (i.e. electronicDipole[10][0][XAxis] is the x-component of the transition dipole moment from the ground state to 10-th excited state.).
    double*      coreDipoleMoment; // dipole moment of configuration.
@@ -111,17 +121,19 @@ protected:
    virtual void CalcSCFProperties();
    virtual void CalcCISProperties();
    virtual void CalcNormalModes(double** normalModes, double* normalForceConstants, const MolDS_base::Molecule& molecule) const;
-   virtual double GetElectronicTransitionDipoleMoment(int to, int from, MolDS_base::CartesianType axis,
-                                                      double const* const* fockMatrix,
-                                                      double const* const* matrixCIS,
-                                                      double const* const* const* cartesianMatrix,
-                                                      const MolDS_base::Molecule& molecule, 
-                                                      double const* const* orbitalElectronPopulation,
-                                                      double const* const* overlapAOs,
-                                                      double const* groundStateDipole) const;
+   virtual void CalcElectronicTransitionDipoleMoment(double* transitionDipoleMoment,
+                                                     int to, int from,
+                                                     double const* const* fockMatrix,
+                                                     double const* const* matrixCIS,
+                                                     double const* const* const* cartesianMatrix,
+                                                     const MolDS_base::Molecule& molecule, 
+                                                     double const* const* orbitalElectronPopulation,
+                                                     double const* const* overlapAOs,
+                                                     double const* groundStateDipole) const;
    double GetBondingAdjustParameterK(MolDS_base::ShellType shellA, 
                                      MolDS_base::ShellType shellB) const;
    virtual double GetDiatomCoreRepulsionEnergy(int indexAtomA, int indexAtomB) const;
+   virtual double GetAtomCoreEpcCoulombEnergy (int indexAtom,  int indexEpc) const;
    virtual double GetDiatomCoreRepulsion1stDerivative(int indexAtomA, 
                                                       int indexAtomB, 
                                                       MolDS_base::CartesianType axisA) const;
@@ -161,7 +173,7 @@ protected:
                                      double const* const* gammaAB,
                                      double const* const* orbitalElectronPopulation, 
                                      double const* atomicElectronPopulation,
-                                     double const* const* const* const* const* const* twoElecTwoCore, 
+                                     double const* const* const* const* const* const* twoElecsTwoAtomCores, 
                                      bool isGuess) const;
    virtual double GetFockOffDiagElement(const MolDS_base_atoms::Atom& atomA, 
                                         const MolDS_base_atoms::Atom& atomB, 
@@ -173,8 +185,9 @@ protected:
                                         double const* const* gammaAB, 
                                         double const* const* overlapAOs,
                                         double const* const* orbitalElectronPopulation, 
-                                        double const* const* const* const* const* const* twoElecTwoCore, 
+                                        double const* const* const* const* const* const* twoElecsTwoAtomCores, 
                                         bool isGuess) const;
+   void TransposeFockMatrixMatrix(double** transposedFockMatrix) const;
    virtual void CalcDiatomicOverlapAOsInDiatomicFrame(double** diatomicOverlapAOs, 
                                                       const MolDS_base_atoms::Atom& atomA, 
                                                       const MolDS_base_atoms::Atom& atomB) const;
@@ -185,15 +198,49 @@ protected:
                                                                    const MolDS_base_atoms::Atom& atomA, 
                                                                    const MolDS_base_atoms::Atom& atomB) const;
    void CalcDiatomicOverlapAOs1stDerivatives(double*** diatomicOverlapAOs1stDerivs, 
+                                             double**  tmpDiaOverlapAOsInDiaFrame,         
+                                             double**  tmpDiaOverlapAOs1stDerivInDiaFrame,
+                                             double**  tmpRotMat, 
+                                             double**  tmpRotMat1stDeriv,
+                                             double*** tmpRotMat1stDerivs,
+                                             double**  tmpRotatedDiatomicOverlap,
+                                             double*   tmpRotatedDiatomicOverlapVec,
+                                             double**  tmpMatrixBC,
+                                             double*   tmpVectorBC,
                                              const MolDS_base_atoms::Atom& atomA, 
                                              const MolDS_base_atoms::Atom& atomB) const;
    void CalcDiatomicOverlapAOs1stDerivatives(double*** diatomicOverlapAOs1stDerivs, 
+                                             double**  tmpDiaOverlapAOsInDiaFrame,         
+                                             double**  tmpDiaOverlapAOs1stDerivInDiaFrame,
+                                             double**  tmpRotMat, 
+                                             double**  tmpRotMat1stDeriv,
+                                             double*** tmpRotMat1stDerivs,
+                                             double**  tmpRotatedDiatomicOverlap,
+                                             double*   tmpRotatedDiatomicOverlapVec,
+                                             double**  tmpMatrixBC,
+                                             double*   tmpVectorBC,
                                              int indexAtomA, 
                                              int indexAtomB) const;
    void CalcDiatomicOverlapAOs2ndDerivatives(double**** overlapAOs2ndDeri, 
+                                             double**   tmpDiaOverlapAOsInDiaFrame,
+                                             double**   tmpDiaOverlapAOs1stDerivInDiaFrame,
+                                             double**   tmpDiaOverlapAOs2ndDerivInDiaFrame,
+                                             double***  tmpDiaOverlapAOs1stDerivs,
+                                             double**** tmpDiaOverlapAOs2ndDerivs,
+                                             double**   tmpRotMat,
+                                             double***  tmpRotMat1stDerivs,
+                                             double**** tmpRotMat2ndDerivs,
                                              const MolDS_base_atoms::Atom& atomA, 
                                              const MolDS_base_atoms::Atom& atomB) const;
    void CalcDiatomicOverlapAOs2ndDerivatives(double**** overlapAOs2ndDeri, 
+                                             double**   tmpDiaOverlapAOsInDiaFrame,
+                                             double**   tmpDiaOverlapAOs1stDerivInDiaFrame,
+                                             double**   tmpDiaOverlapAOs2ndDerivInDiaFrame,
+                                             double***  tmpDiaOverlapAOs1stDerivs,
+                                             double**** tmpDiaOverlapAOs2ndDerivs,
+                                             double**   tmpRotMat,
+                                             double***  tmpRotMat1stDerivs,
+                                             double**** tmpRotMat2ndDerivs,
                                              int indexAtomA, 
                                              int indexAtomB) const;
    double Get2ndDerivativeElementFromDistanceDerivatives(double firstDistanceDeri,
@@ -201,13 +248,14 @@ protected:
                                                          MolDS_base::CartesianType axisA1,
                                                          MolDS_base::CartesianType axisA2,
                                                          double* cartesian,
-                                                         double Rab) const;
+                                                         double rAB) const;
    virtual double GetMolecularIntegralElement(int moI, int moJ, int moK, int moL, 
                                               const MolDS_base::Molecule& molecule, 
                                               double const* const* fockMatrix, 
                                               double const* const* gammaAB) const;
-   virtual void CalcTwoElecTwoCore(double****** twoElecTwoCore, 
-                                   const MolDS_base::Molecule& molecule) const;
+   virtual void CalcTwoElecsTwoCores(double****** twoElecsTwoAtomCores, 
+                                     double****** twoElecsAtomEpcCores,
+                                     const MolDS_base::Molecule& molecule) const;
    virtual void CalcForce(const std::vector<int>& elecStates);
    void CalcRotatingMatrix1stDerivatives(double*** rotMat1stDerivatives, 
                                          const MolDS_base_atoms::Atom& atomA,
@@ -240,26 +288,34 @@ private:
    std::string errorMessageCalcRotatingMatrixNullRotMatrix;
    std::string errorMessageRotDiaOverlapAOsToSpaceFrameNullDiaMatrix;
    std::string errorMessageRotDiaOverlapAOsToSpaceFrameNullRotMatrix;
+   std::string errorMessageRotDiaOverlapAOsToSpaceFrameNullTmpDiaMatrix;
+   std::string errorMessageRotDiaOverlapAOsToSpaceFrameNullTmpOldDiaMatrix;
+   std::string errorMessageRotDiaOverlapAOsToSpaceFrameNullTmpMatrixBC;
+   std::string errorMessageRotDiaOverlapAOsToSpaceFrameNullTmpVectorBC;
    std::string errorMessageSetOverlapAOsElementNullDiaMatrix;
    std::string errorMessageCalcOverlapAOsDifferentConfigurationsDiffAOs;
    std::string errorMessageCalcOverlapAOsDifferentConfigurationsDiffAtoms;
    std::string errorMessageCalcOverlapAOsDifferentConfigurationsOverlapAOsNULL;
    std::string errorMessageLhs;
    std::string errorMessageRhs;
+   std::string messageIterSCFTitle;
    std::string messageIterSCF;
-   std::string messageDensityRMS;
+   std::string messageDiisApplied;
+   std::string messageDampingApplied;
    std::string messageEnergyMO;
    std::string messageEnergyMOTitle;
-   std::string messageMullikenAtoms;
-   std::string messageMullikenAtomsTitle;
    std::string messageElecEnergy;
    std::string messageNoteElecEnergy;
    std::string messageNoteElecEnergyVdW;
+   std::string messageNoteElecEnergyEpcVdW;
+   std::string messageNoteElecEnergyEpc;
    std::string messageElecEnergyTitle;
    std::string messageOcc;
    std::string messageUnOcc;
    std::string messageCoreRepulsionTitle;
    std::string messageCoreRepulsion;
+   std::string messageCoreEpcCoulombTitle;
+   std::string messageCoreEpcCoulomb;
    std::string messageVdWCorrectionTitle;
    std::string messageVdWCorrection;
    std::string messageElectronicDipoleMomentTitle;
@@ -268,6 +324,7 @@ private:
    std::string messageCoreDipoleMoment;
    std::string messageTotalDipoleMomentTitle;
    std::string messageTotalDipoleMoment;
+   std::string messageMullikenAtomsSCF;
    std::string messageNormalModesTitle;
    std::string messageNormalModesUnitsMassWeighted;
    std::string messageNormalModesUnitsNonMassWeighted;
@@ -315,9 +372,12 @@ private:
                                               double const* const* overlapAOs) const;
    bool SatisfyConvergenceCriterion(double const* const* oldOrbitalElectronPopulation, 
                                     double const* const* orbitalElectronPopulation,
-                                    int numberAOs, 
+                                    int     numberAOs, 
                                     double* rmsDensity, 
-                                    int times) const;
+                                    int     times,
+                                    double  diisErro,
+                                    bool    hasAppliedDIIS,
+                                    bool    hasAppliedDamping) const;
    void UpdateOldOrbitalElectronPopulation(double** oldOrbitalElectronPopulation, 
                                            double const* const* orbitalElectronPopulation,
                                            int numberAOs) const;
@@ -332,11 +392,13 @@ private:
    void CalcCartesianMatrixByGTOExpansion(double*** cartesianMatrix,
                                           const MolDS_base::Molecule& molecule, 
                                           MolDS_base::STOnGType stonG) const; 
-   double GetCartesianMatrixElementByGTOExpansion(const MolDS_base_atoms::Atom& atomA, 
+   void CalcCartesianMatrixElementsByGTOExpansion(double& xComponent,
+                                                  double& yComponent,
+                                                  double& zComponent,
+                                                  const MolDS_base_atoms::Atom& atomA, 
                                                   int valenceIndexA, 
                                                   const MolDS_base_atoms::Atom& atomB, 
                                                   int valenceIndexB,
-                                                  MolDS_base::CartesianType axis,
                                                   MolDS_base::STOnGType stonG) const;
    double GetGaussianCartesianMatrix(MolDS_base::AtomType atomTypeA, 
                                      MolDS_base::OrbitalType valenceOrbitalA, 
@@ -346,7 +408,18 @@ private:
                                      MolDS_base::OrbitalType valenceOrbitalB, 
                                      double gaussianExponentB, 
                                      double const* xyzB,
-                                     double Rab,
+                                     double rAB,
+                                     MolDS_base::CartesianType axis) const;
+   double GetGaussianCartesianMatrix(MolDS_base::AtomType atomTypeA, 
+                                     MolDS_base::OrbitalType valenceOrbitalA, 
+                                     double gaussianExponentA, 
+                                     double const* xyzA,
+                                     MolDS_base::AtomType atomTypeB, 
+                                     MolDS_base::OrbitalType valenceOrbitalB, 
+                                     double gaussianExponentB, 
+                                     double const* xyzB,
+                                     double rAB,
+                                     double ovelapSASB,
                                      MolDS_base::CartesianType axis) const;
    void CalcOverlapAOs(double** overlapAOs, const MolDS_base::Molecule& molecule) const;
    void CalcOverlapAOsByGTOExpansion(double** overlapAOs, 
@@ -357,9 +430,9 @@ private:
                                              const MolDS_base_atoms::Atom& atomB, 
                                              int valenceIndexB,
                                              MolDS_base::STOnGType stonG) const; // see [DY_1977]
-   double GetGaussianOverlapAOsSaSb(double gaussianExponentA, 
+   double GetGaussianOverlapAOsSASB(double gaussianExponentA, 
                                     double gaussianExponentB, 
-                                    double Rab) const; // see [DY_1977]
+                                    double rAB) const; // see [DY_1977]
    double GetGaussianOverlapAOs(MolDS_base::AtomType atomTypeA, 
                                 MolDS_base::OrbitalType valenceOrbitalA, 
                                 double gaussianExponentA, 
@@ -369,7 +442,18 @@ private:
                                 double dx, 
                                 double dy, 
                                 double dz, 
-                                double Rab) const; // see [DY_1977]
+                                double rAB) const; // see [DY_1977]
+   double GetGaussianOverlapAOs(MolDS_base::AtomType atomTypeA, 
+                                MolDS_base::OrbitalType valenceOrbitalA, 
+                                double gaussianExponentA, 
+                                MolDS_base::AtomType atomTypeB, 
+                                MolDS_base::OrbitalType valenceOrbitalB, 
+                                double gaussianExponentB, 
+                                double dx, 
+                                double dy, 
+                                double dz, 
+                                double rAB,
+                                double ovelapSASB) const; // see [DY_1977]
    double GetGaussianOverlapAOs1stDerivative(MolDS_base::AtomType atomTypeA, 
                                              MolDS_base::OrbitalType valenceOrbitalA, 
                                              double gaussianExponentA, 
@@ -379,7 +463,7 @@ private:
                                              double dx, 
                                              double dy, 
                                              double dz, 
-                                             double Rab, 
+                                             double rAB, 
                                              MolDS_base::CartesianType axisA) const;// see [DY_1977]
    void CalcFockMatrix(double** fockMatrix, 
                        const MolDS_base::Molecule& molecule, 
@@ -387,10 +471,14 @@ private:
                        double const* const* gammaAB,
                        double const* const* orbitalElectronPopulation, 
                        double const* atomicElectronPopulation,
-                       double const* const* const* const* const* const* twoElecTwoCore,
+                       double const* const* const* const* const* const* twoElecsTwoAtomCores,
                        bool isGuess) const;
-   void RotateDiatmicOverlapAOsToSpaceFrame(double** diatomicOverlapAOs, 
-                                            double const* const* rotatingMatrix) const;
+   void RotateDiatmicOverlapAOsToSpaceFrame(double**             diatomicOverlapAOs, 
+                                            double const* const* rotatingMatrix,
+                                            double*              tmpDiatomicOverlapAOs,
+                                            double**             tmpOldDiatomicOverlapAOs,
+                                            double**             tmpMatrixBC,
+                                            double*              tmpVectorBC) const;
    void SetOverlapAOsElement(double** overlapAOs, 
                              double const* const* diatomicOverlapAOs, 
                              const MolDS_base_atoms::Atom& atomA, 
@@ -404,6 +492,7 @@ private:
    double GetAuxiliaryB1stDerivative(int k, double rho) const;
    double GetAuxiliaryB2ndDerivative(int k, double rho) const;
    void DoDamp(double rmsDensity, 
+               bool&  hasAppliedDamping,
                double** orbitalElectronPopulation, 
                double const* const* oldOrbitalElectronPopulation, 
                const MolDS_base::Molecule& molecule) const;
@@ -412,7 +501,10 @@ private:
                double*** diisStoredDensityMatrix,
                double*** diisStoredErrorVect,
                double**  diisErrorProducts,
+               double**  tmpDiisErrorProducts,
                double*   diisErrorCoefficients,
+               double&   diisError,
+               bool&     hasAppliedDIIS,
                int       diisNumErrorVect,
                const MolDS_base::Molecule& molecule, 
                int step) const;
@@ -426,6 +518,7 @@ private:
                           double const* const* fockMatrix, 
                           double const* const* gammaAB, 
                           double coreRepulsionEnergy,
+                          double coreEpcCoulombEnergy,
                           double vdWCorrectionEnergy) const;
    void FreeElecEnergyMatrices(double*** fMatrix, 
                                double*** hMatrix, 
@@ -435,39 +528,16 @@ private:
                                  double**** diisStoredDensityMatrix,
                                  double**** diisStoredErrorVect,
                                  double***  diisErrorProducts,
+                                 double***  tmpDiisErrorProducts,
                                  double**   diisErrorCoefficients) const;
    void MallocSCFTemporaryMatrices(double***  oldOrbitalElectronPopulation,
                                    double**** diisStoredDensityMatrix,
                                    double**** diisStoredErrorVect,
                                    double***  diisErrorProducts,
+                                   double***  tmpDiisErrorProducts,
                                    double**   diisErrorCoefficients);
-   void MallocDiatomicOverlapAOs1stDeriTemps(double***  diaOverlapAOsInDiaFrame, 
-                                             double***  diaOverlapAOs1stDerivInDiaFrame,
-                                             double***  rotMat,
-                                             double**** rotMat1stDerivs) const;
-   void MallocDiatomicOverlapAOs2ndDeriTemps(double***   diaOverlapAOsInDiaFrame, 
-                                             double***   diaOverlapAOs1stDerivInDiaFrame,
-                                             double***   diaOverlapAOs2ndDerivInDiaFrame,
-                                             double***   rotMat,
-                                             double****  rotMat1stDerivs,
-                                             double***** rotMat2ndDerivs,
-                                             double****  tempDiaOverlapAOs1stDerivs,
-                                             double***** tempDiaOverlapAOs2ndDerivs) const;
-   void FreeDiatomicOverlapAOs1stDeriTemps(double***  diaOverlapAOsInDiaFrame, 
-                                           double***  diaOverlapAOs1stDerivInDiaFrame,
-                                           double***  rotMat,
-                                           double**** rotMat1stDerivs) const;
-   void FreeDiatomicOverlapAOs2ndDeriTemps(double***   diaOverlapAOsInDiaFrame, 
-                                           double***   diaOverlapAOs1stDerivInDiaFrame,
-                                           double***   diaOverlapAOs2ndDerivInDiaFrame,
-                                           double***   rotMat,
-                                           double****  rotMat1stDerivs,
-                                           double***** rotMat2ndDerivs,
-                                           double****  tempDiaOverlapAOs1stDerivs,
-                                           double***** tempDiaOverlapAOs2ndDerivs) const;
 };
 
-
 }
 #endif
 
