@@ -21,6 +21,7 @@
 #include<stdlib.h>
 #include<string.h>
 #include<iostream>
+#include<algorithm>
 #include<sstream>
 #include<math.h>
 #include<string>
@@ -28,15 +29,19 @@
 #include<stdexcept>
 #include<boost/shared_ptr.hpp>
 #include<boost/format.hpp>
+#include"../config.h"
+#include"../base/Enums.h"
+#include"../base/Uncopyable.h"
 #include"../base/PrintController.h"
 #include"../base/MolDSException.h"
-#include"../base/Uncopyable.h"
+#include"../base/MallocerFreer.h"
+#include"../mpi/MpiInt.h"
+#include"../mpi/MpiProcess.h"
 #include"../wrappers/Blas.h"
 #include"../wrappers/Lapack.h"
-#include"../base/Enums.h"
-#include"../base/MallocerFreer.h"
 #include"../base/EularAngle.h"
 #include"../base/Parameters.h"
+#include"../base/RealSphericalHarmonicsIndex.h"
 #include"../base/atoms/Atom.h"
 #include"../base/Molecule.h"
 #include"../base/ElectronicStructure.h"
@@ -105,10 +110,10 @@ void BFGS::SearchMinimum(boost::shared_ptr<ElectronicStructure> electronicStruct
    int totalSteps = Parameters::GetInstance()->GetTotalStepsOptimization();
    double maxGradientThreshold = Parameters::GetInstance()->GetMaxGradientOptimization();
    double rmsGradientThreshold = Parameters::GetInstance()->GetRmsGradientOptimization();
-   double lineSearchCurrentEnergy = 0.0;
-   double lineSearchInitialEnergy = 0.0;
-   double** matrixForce = NULL;
-   double* vectorForce = NULL;
+   double lineSearchCurrentEnergy   = 0.0;
+   double lineSearchInitialEnergy   = 0.0;
+   double const* const* matrixForce = NULL;
+   double const* vectorForce        = NULL;
    const int dimension = molecule.GetNumberAtoms()*CartesianType_end;
    double** matrixHessian        = NULL;
    double*  vectorOldForce       = NULL;
@@ -123,11 +128,8 @@ void BFGS::SearchMinimum(boost::shared_ptr<ElectronicStructure> electronicStruct
    try{
       // initialize Hessian with unit matrix
       MallocerFreer::GetInstance()->Malloc(&matrixHessian, dimension, dimension);
-      for(int i=0; i<dimension; i++){
-         for(int j=i; j<dimension; j++){
-            matrixHessian[i][j] = matrixHessian[j][i] = i==j ? 1.0 : 0.0;
-         }
-      }
+      const double one = 1;
+      MolDS_wrappers::Blas::GetInstance()->Dcopy(dimension, &one, 0, &matrixHessian[0][0], dimension+1);
 
       // initial calculation
       bool requireGuess = true;
@@ -143,19 +145,10 @@ void BFGS::SearchMinimum(boost::shared_ptr<ElectronicStructure> electronicStruct
 
          // Store old Force data
          MallocerFreer::GetInstance()->Malloc(&vectorOldForce, dimension);
-         for(int i =0;i < dimension; i++){
-            vectorOldForce[i] = vectorForce[i];
-         }
+         MolDS_wrappers::Blas::GetInstance()->Dcopy(dimension, vectorForce, vectorOldForce);
+
+         this->StoreMolecularGeometry(matrixOldCoordinates, molecule);
 
-         //Store old coordinates
-         MallocerFreer::GetInstance()->Malloc(&matrixOldCoordinates, molecule.GetNumberAtoms(), CartesianType_end);
-         for(int i=0;i<molecule.GetNumberAtoms();i++){
-            const Atom*   atom = molecule.GetAtom(i);
-            const double* xyz  = atom->GetXyz();
-            for(int j=0;j<CartesianType_end;j++){
-               matrixOldCoordinates[i][j] = xyz[j];
-            }
-         }
          // Level shift Hessian redundant modes
          this->ShiftHessianRedundantMode(matrixHessian, molecule);
 
@@ -167,19 +160,7 @@ void BFGS::SearchMinimum(boost::shared_ptr<ElectronicStructure> electronicStruct
          vectorStep = &matrixStep[0][0];
          this->CalcRFOStep(vectorStep, matrixHessian, vectorForce, trustRadius, dimension);
 
-         // Calculate approximate change of energy using
-         // [2/2] Pade approximant
-         // See Eq. (2) in [BB_1998]
-         double approximateChangeNumerator   = 0;
-         double approximateChangeDenominator = 1;
-         for(int i=0;i<dimension;i++){
-            approximateChangeNumerator -= vectorForce[i] * vectorStep[i];
-            approximateChangeDenominator += vectorStep[i] * vectorStep[i];
-            for(int j=0;j<dimension;j++){
-               approximateChangeNumerator += vectorStep[i] * matrixHessian[i][j] * vectorStep[j] / 2;
-            }
-         }
-         double approximateChange = approximateChangeNumerator / approximateChangeDenominator;
+         double approximateChange = this->ApproximateEnergyChange(dimension, matrixHessian, vectorForce, vectorStep);
 
          // Take a RFO step
          bool doLineSearch = false;
@@ -190,20 +171,17 @@ void BFGS::SearchMinimum(boost::shared_ptr<ElectronicStructure> electronicStruct
          }
          else{
             this->UpdateMolecularCoordinates(molecule, matrixStep);
+
+            // Broadcast to all processes
+            int root = MolDS_mpi::MpiProcess::GetInstance()->GetHeadRank();
+            molecule.BroadcastConfigurationToAllProcesses(root);
+
             this->UpdateElectronicStructure(electronicStructure, molecule, requireGuess, tempCanOutputLogs);
             lineSearchCurrentEnergy = electronicStructure->GetElectronicEnergy(elecState);
          }
          this->OutputMoleculeElectronicStructure(electronicStructure, molecule, this->CanOutputLogs());
 
-         // Calculate the correctness of the approximation
-         double r = (lineSearchCurrentEnergy - lineSearchInitialEnergy)
-            / approximateChange; // correctness of the step
-         bool aproxcheckCanOutputLogs = true;
-         tempCanOutputLogs = molecule.CanOutputLogs();
-         molecule.SetCanOutputLogs(aproxcheckCanOutputLogs);
-         this->OutputLog(boost::format(this->formatEnergyChangeComparison)
-               % (lineSearchCurrentEnergy-lineSearchInitialEnergy) % approximateChange % r);
-         molecule.SetCanOutputLogs(tempCanOutputLogs);
+         this->UpdateTrustRadius(trustRadius, approximateChange, lineSearchInitialEnergy, lineSearchCurrentEnergy);
 
          // check convergence
          if(this->SatisfiesConvergenceCriterion(matrixForce,
@@ -216,49 +194,15 @@ void BFGS::SearchMinimum(boost::shared_ptr<ElectronicStructure> electronicStruct
             break;
          }
 
-         // Update the trust radius
-         if(r < 0)
-         {
-            // Rollback molecular geometry
-            bool tempCanOutputLogs = molecule.CanOutputLogs();
-            bool rollbackCanOutputLogs = true;
-            molecule.SetCanOutputLogs(rollbackCanOutputLogs);
+         if(lineSearchCurrentEnergy > lineSearchInitialEnergy){
             this->OutputLog(this->messageHillClimbing);
-            for(int i=0;i<molecule.GetNumberAtoms();i++){
-               const Atom* atom = molecule.GetAtom(i);
-               double*     xyz  = atom->GetXyz();
-               for(int j=0;j<CartesianType_end;j++){
-                  xyz[j] = matrixOldCoordinates[i][j];
-               }
-            }
+            this->RollbackMolecularGeometry(molecule, matrixOldCoordinates);
             lineSearchCurrentEnergy = lineSearchInitialEnergy;
-            molecule.SetCanOutputLogs(tempCanOutputLogs);
-            // and rerun with smaller trust radius
-            // without updating Hessian
-            trustRadius /= 4;
-            continue;
-         }
-         else if(r<0.25){
-            trustRadius /= 4;
-         }
-         else if(r<0.75){
-            // keep trust radius
-         }
-         else if(r<2){
-            trustRadius *= 2;
-         }
-         else{
-            trustRadius /= 2;
          }
+
          //Calculate displacement (K_k at Eq. (15) in [SJTO_1983])
-         MallocerFreer::GetInstance()->Malloc(&matrixDisplacement, molecule.GetNumberAtoms(), CartesianType_end);
-         for(int i=0;i<molecule.GetNumberAtoms();i++){
-            const Atom*   atom = molecule.GetAtom(i);
-            const double* xyz  = atom->GetXyz();
-            for(int j=0;j<CartesianType_end;j++){
-               matrixDisplacement[i][j] = xyz[j] - matrixOldCoordinates[i][j];
-            }
-         }
+         this->CalcDisplacement(matrixDisplacement, matrixOldCoordinates, molecule);
+
          matrixForce = electronicStructure->GetForce(elecState);
          vectorForce = &matrixForce[0][0];
 
@@ -410,16 +354,20 @@ void BFGS::UpdateHessian(double **matrixHessian,
 
 void BFGS::ShiftHessianRedundantMode(double** matrixHessian,
                                      const Molecule& molecule) const{
-   const double largeEigenvalue       = 1.0e3;
-   const int    numAtoms              = molecule.GetNumberAtoms();
-   const int    dimension             = numAtoms *CartesianType_end;
-   const int    numTranslationalModes = 3;
-   const int    numRotationalModes    = 3;
-   int          numRedundantModes     = numTranslationalModes + numRotationalModes;
-   double** vectorsHessianModes       = NULL;
-   double*  vectorHessianEigenValues  = NULL;
-   double** matrixesRedundantModes[]  = {NULL, NULL, NULL, NULL, NULL, NULL};
-   double*   vectorsRedundantModes[]  = {NULL, NULL, NULL, NULL, NULL, NULL};
+   const double one                      = 1;
+   const double largeEigenvalue          = 1.0e3;
+   const int    numAtoms                 = molecule.GetNumberAtoms();
+   const int    dimension                = numAtoms *CartesianType_end;
+   const int    numTranslationalModes    = 3;
+   const int    numRotationalModes       = 3;
+   int          numRedundantModes        = numTranslationalModes + numRotationalModes;
+   double** vectorsHessianModes          = NULL;
+   double*  vectorHessianEigenValues     = NULL;
+   double** matrixesRedundantModes[]     = {NULL, NULL, NULL, NULL, NULL, NULL};
+   double*  vectorsRedundantModes[]      = {NULL, NULL, NULL, NULL, NULL, NULL};
+   double** matrixProjection             = NULL;
+   double*  vectorProjectedRedundantMode = NULL;
+   double** matrixShiftedHessianBuffer   = NULL;
    const double matrixesRotationalModeGenerators[numRotationalModes]
                                                 [CartesianType_end]
                                                 [CartesianType_end]
@@ -434,7 +382,7 @@ void BFGS::ShiftHessianRedundantMode(double** matrixHessian,
          vectorsRedundantModes[c] = &matrixesRedundantModes[c][0][0];
       }
       for(int c=0; c<numTranslationalModes;c++){
-#pragma omp parallel for schedule(auto)
+#pragma omp parallel for schedule(dynamic, MOLDS_OMP_DYNAMIC_CHUNK_SIZE)
          for(int n=0;n<numAtoms;n++){
             for(int d=0;d<CartesianType_end;d++){
                matrixesRedundantModes[c][n][d] = c==d? 1.0 : 0.0;
@@ -447,7 +395,7 @@ void BFGS::ShiftHessianRedundantMode(double** matrixHessian,
          vectorsRedundantModes[c+numTranslationalModes] = &matrixesRedundantModes[c+numTranslationalModes][0][0];
       }
       for(int c=0; c<numRotationalModes;c++){
-#pragma omp parallel for schedule(auto)
+#pragma omp parallel for schedule(dynamic, MOLDS_OMP_DYNAMIC_CHUNK_SIZE)
          for(int n=0;n<numAtoms;n++){
             const double* xyz = molecule.GetAtom(n)->GetXyz();
             for(int d=0;d<CartesianType_end;d++){
@@ -458,150 +406,182 @@ void BFGS::ShiftHessianRedundantMode(double** matrixHessian,
             }
          }
       }
-      // Orthonormalize redundant modes
-      // using Gram-Schmidt process
-      for(int c=0; c<numRedundantModes;c++){
-         for(int d=0;d<c;d++){
-            double dotproduct = 0.0;
-#pragma omp parallel for schedule(auto) reduction(+:dotproduct)
-            for(int i=0;i<dimension;i++){
-               dotproduct += vectorsRedundantModes[d][i] * vectorsRedundantModes[c][i];
-            }
-#pragma omp parallel for schedule(auto)
-            for(int i=0;i<dimension;i++){
-               vectorsRedundantModes[c][i] -= dotproduct * vectorsRedundantModes[d][i];
-            }
-         }
-         double norm = 0.0;
-#pragma omp parallel for schedule(auto) reduction(+:norm)
-         for(int i=0;i<dimension;i++){
-            norm += vectorsRedundantModes[c][i] * vectorsRedundantModes[c][i];
-         }
-         norm = sqrt(norm);
-         // Eliminate a linear dependent mode
-         if(norm < 1e-5){
-            numRedundantModes--;
-            for(int d=c;d<numRedundantModes;d++){
-#pragma omp parallel for schedule(auto)
-               for(int i=0;i<dimension;i++){
-                  vectorsRedundantModes[d][i] = vectorsRedundantModes[d+1][i];
-               }
-            }
-         }
-         else{
-#pragma omp parallel for schedule(auto)
-            for(int i=0;i<dimension;i++){
-               vectorsRedundantModes[c][i] /= norm;
-            }
-         }
+      //Prepare the identity matrix I
+      MallocerFreer::GetInstance()->Malloc(&matrixProjection, dimension, dimension);
+      MolDS_wrappers::Blas::GetInstance()->Dcopy(dimension, &one, 0, &matrixProjection[0][0], dimension+1);
+
+      //Prepare the projection matrix P = I - sum_i u_i u_i^T, that projects redundant modes to 0 vector
+      MallocerFreer::GetInstance()->Malloc(&vectorProjectedRedundantMode, dimension);
+      for(int c=0; c<numRedundantModes; c++){
+         double normSquare = 0;
+         MolDS_wrappers::Blas::GetInstance()->Dsymv(dimension, matrixProjection, vectorsRedundantModes[c], vectorProjectedRedundantMode);
+         normSquare = MolDS_wrappers::Blas::GetInstance()->Ddot(dimension, vectorProjectedRedundantMode, vectorProjectedRedundantMode);
+         MolDS_wrappers::Blas::GetInstance()->Dsyr(dimension, -1.0/normSquare, vectorProjectedRedundantMode, matrixProjection);
       }
 
-      // Diagonalize hessian
-      MallocerFreer::GetInstance()->Malloc(&vectorHessianEigenValues, dimension);
-      MallocerFreer::GetInstance()->Malloc(&vectorsHessianModes, dimension, dimension);
-#pragma omp parallel for schedule(auto)
-      for(int i=0; i<dimension; i++){
-         for(int j=0; j<dimension; j++){
-            vectorsHessianModes[i][j] = matrixHessian[i][j];
-         }
-      }
-      bool calcEigenVectors = true;
-      MolDS_wrappers::Lapack::GetInstance()->Dsyevd(&vectorsHessianModes[0],
-                                                    &vectorHessianEigenValues[0],
-                                                    dimension,
-                                                    calcEigenVectors);
+      //// Diagonalize hessian
+      //MallocerFreer::GetInstance()->Malloc(&vectorHessianEigenValues, dimension);
+      //MallocerFreer::GetInstance()->Malloc(&vectorsHessianModes, dimension, dimension);
+      //MolDS_wrappers::Blas::GetInstance()->Dcopy(dimension*dimension, &matrixHessian[0][0], &vectorsHessianModes[0][0]);
+      //bool calcEigenVectors = true;
+      //MolDS_wrappers::Lapack::GetInstance()->Dsyevd(&vectorsHessianModes[0],
+      //                                              &vectorHessianEigenValues[0],
+      //                                              dimension,
+      //                                              calcEigenVectors);
+      //
+      //// Output eigenvalues of the raw Hessian to the log
+      //this->OutputLog(this->messageRawHessianEigenvalues);
+      //for(int i=0;i<dimension;i++){
+      //   if((i%6) == 0){
+      //      this->OutputLog(boost::format("\n%e")%vectorHessianEigenValues[i]);
+      //   }
+      //   else{
+      //      this->OutputLog(boost::format(",\t%e")%vectorHessianEigenValues[i]);
+      //   }
+      //}
+      //this->OutputLog("\n");
 
-      // Output eigenvalues of the raw Hessianto the log
-      this->OutputLog(this->messageRawHessianEigenvalues);
-      for(int i=0;i<dimension;i++){
-         if((i%6) == 0){
-            this->OutputLog(boost::format("\n%e")%vectorHessianEigenValues[i]);
-         }
-         else{
-            this->OutputLog(boost::format(",\t%e")%vectorHessianEigenValues[i]);
-         }
-      }
-      this->OutputLog("\n");
+      // Project Hessian H' = P H P
+      MallocerFreer::GetInstance()->Malloc(&matrixShiftedHessianBuffer, dimension, dimension);
+      // TODO: Use dsymm instead of dgemm
+      MolDS_wrappers::Blas::GetInstance()->Dgemm(dimension, dimension, dimension,
+            matrixProjection            , matrixHessian   , matrixShiftedHessianBuffer);
+      MolDS_wrappers::Blas::GetInstance()->Dgemm(dimension, dimension, dimension,
+            matrixShiftedHessianBuffer, matrixProjection, matrixHessian);
 
-      // Orthogonalize vectorsHessianModes against vectorsRedundantModes
-#pragma omp parallel for schedule(auto)
-      for(int i=0;i<dimension-numRedundantModes;i++){
-         for(int c=0; c<numRedundantModes;c++){
-            double dotproduct = 0.0;
-            for(int j=0;j<dimension;j++){
-               dotproduct += vectorsHessianModes[i][j] * vectorsRedundantModes[c][j];
-            }
-            for(int j=0;j<dimension;j++){
-               vectorsHessianModes[i][j] -= dotproduct * vectorsRedundantModes[c][j];
-            }
-            double norm = 0.0;
-            for(int j=0;j<dimension;j++){
-               norm += vectorsHessianModes[i][j] * vectorsHessianModes[i][j];
-            }
-            norm = sqrt(norm);
-            for(int j=0;j<dimension;j++){
-               vectorsHessianModes[i][j] /= norm;
-            }
-         }
-      }
+      // Shift eigenvalues for redundant mode by adding L sum_i u_i*u_i^T = L ( I - P )= -L (P - I)
+      MolDS_wrappers::Blas::GetInstance()->Daxpy(dimension, -1, &one, 0, &matrixProjection[0][0], dimension + 1); // (P - I)
+      MolDS_wrappers::Blas::GetInstance()->Daxpy(dimension*dimension, -largeEigenvalue, &matrixProjection[0][0], &matrixHessian[0][0]);
 
-      // Calculate projected eigenvalues of new modes
-      // h_i' = x_i'^T * H * x_i'
-#pragma omp parallel for schedule(auto)
-      for(int i=0;i<dimension-numRedundantModes;i++){
-         double tmp = 0.0;
-         for(int j=0;j<dimension;j++){
-            for(int k=0;k<dimension;k++){
-               tmp += vectorsHessianModes[i][j] * matrixHessian[j][k] * vectorsHessianModes[i][k];
-            }
-         }
-         vectorHessianEigenValues[i] = tmp;
-      }
-#pragma omp parallel for schedule(auto)
-      for(int i=dimension-numRedundantModes;i<dimension;i++){
-         vectorHessianEigenValues[i] = largeEigenvalue;
-      }
-
-      // Output eigenvalues of the raw Hessianto the log
-      this->OutputLog(this->messageShiftedHessianEigenvalues);
-      for(int i=0;i<dimension;i++){
-         if((i%6) == 0){
-            this->OutputLog(boost::format("\n%e")%vectorHessianEigenValues[i]);
-         }
-         else{
-            this->OutputLog(boost::format(",\t%e")%vectorHessianEigenValues[i]);
-         }
-      }
-      this->OutputLog("\n");
-
-      // Calculate shifted Hessian from eigenvalues and modes
-      // H' = sum x_i' h_i' x_i^T
-#pragma omp parallel for schedule(auto)
-      for(int i=0;i<dimension;i++){
-         for(int j=0;j<dimension;j++){
-            double tmp = 0.0;
-            for(int k=0;k<dimension-numRedundantModes;k++){
-               tmp += vectorsHessianModes[k][i] * vectorsHessianModes[k][j] * vectorHessianEigenValues[k];
-            }
-            for(int k=0;k<numRedundantModes;k++){
-               tmp += vectorsRedundantModes[k][i] * vectorsRedundantModes[k][j] * largeEigenvalue;
-            }
-            matrixHessian[i][j] = tmp;
-         }
-      }
+      //// Diagonalize shifted hessian
+      //MolDS_wrappers::Blas::GetInstance()->Dcopy(dimension*dimension, &matrixHessian[0][0], &vectorsHessianModes[0][0]);
+      //calcEigenVectors = true;
+      //MolDS_wrappers::Lapack::GetInstance()->Dsyevd(&vectorsHessianModes[0],
+      //                                              &vectorHessianEigenValues[0],
+      //                                              dimension,
+      //                                              calcEigenVectors);
+      //
+      //// Output eigenvalues of the shifted Hessian to the log
+      //this->OutputLog(this->messageShiftedHessianEigenvalues);
+      //for(int i=0;i<dimension;i++){
+      //   if((i%6) == 0){
+      //      this->OutputLog(boost::format("\n%e")%vectorHessianEigenValues[i]);
+      //   }
+      //   else{
+      //      this->OutputLog(boost::format(",\t%e")%vectorHessianEigenValues[i]);
+      //   }
+      //}
+      //this->OutputLog("\n");
    }
    catch(MolDSException ex)
    {
       for(int i=0;i<numRedundantModes;i++){
          MallocerFreer::GetInstance()->Free(&matrixesRedundantModes[i], numAtoms, CartesianType_end);
       }
+      MallocerFreer::GetInstance()->Free(&matrixProjection, dimension, dimension);
+      MallocerFreer::GetInstance()->Free(&vectorProjectedRedundantMode, dimension);
+      MallocerFreer::GetInstance()->Free(&matrixShiftedHessianBuffer, dimension, dimension);
       MallocerFreer::GetInstance()->Free(&vectorHessianEigenValues, dimension);
       MallocerFreer::GetInstance()->Free(&vectorsHessianModes, dimension, dimension);
    }
    for(int i=0;i<numRedundantModes;i++){
       MallocerFreer::GetInstance()->Free(&matrixesRedundantModes[i], numAtoms, CartesianType_end);
    }
+   MallocerFreer::GetInstance()->Free(&matrixProjection, dimension, dimension);
+   MallocerFreer::GetInstance()->Free(&vectorProjectedRedundantMode, dimension);
+   MallocerFreer::GetInstance()->Free(&matrixShiftedHessianBuffer, dimension, dimension);
    MallocerFreer::GetInstance()->Free(&vectorHessianEigenValues, dimension);
    MallocerFreer::GetInstance()->Free(&vectorsHessianModes, dimension, dimension);
 }
+
+double BFGS::ApproximateEnergyChange(int dimension,
+                                     double const* const* matrixHessian,
+                                     double const* vectorForce,
+                                     double const* vectorStep) const{
+   // Calculate approximate change of energy using
+   // [2/2] Pade approximant
+   // See Eq. (2) in [BB_1998]
+   double approximateChangeNumerator   = 0;
+   double approximateChangeDenominator = 1;
+   for(int i=0;i<dimension;i++){
+      approximateChangeNumerator -= vectorForce[i] * vectorStep[i];
+      approximateChangeDenominator += vectorStep[i] * vectorStep[i];
+      for(int j=0;j<dimension;j++){
+         approximateChangeNumerator += vectorStep[i] * matrixHessian[i][j] * vectorStep[j] / 2;
+      }
+   }
+   return approximateChangeNumerator / approximateChangeDenominator;
+}
+
+void BFGS::UpdateTrustRadius(double &trustRadius,
+                       double approximateEnergyChange,
+                       double initialEnergy,
+                       double currentEnergy)const{
+   // Calculate the correctness of the approximation
+   double r = (currentEnergy - initialEnergy)
+            / approximateEnergyChange;
+
+   this->OutputLog(boost::format(this->formatEnergyChangeComparison)
+                   % (currentEnergy-initialEnergy) % approximateEnergyChange % r);
+
+   if(r < 0)
+   {
+      trustRadius /= 4;
+   }
+   else if(r<0.25){
+      trustRadius /= 4;
+   }
+   else if(r<0.75){
+      // keep trust radius
+   }
+   else if(r<2){
+      trustRadius *= 2;
+   }
+   else{
+      trustRadius /= 2;
+   }
+}
+
+void BFGS::RollbackMolecularGeometry(MolDS_base::Molecule& molecule,
+                                     double const* const* matrixOldCoordinates)const{
+   // Rollback molecular geometry
+   bool tempCanOutputLogs = molecule.CanOutputLogs();
+   bool rollbackCanOutputLogs = true;
+   molecule.SetCanOutputLogs(rollbackCanOutputLogs);
+   for(int i=0;i<molecule.GetNumberAtoms();i++){
+      const Atom* atom = molecule.GetAtom(i);
+      double*     xyz  = atom->GetXyz();
+      for(int j=0;j<CartesianType_end;j++){
+         xyz[j] = matrixOldCoordinates[i][j];
+      }
+   }
+   molecule.SetCanOutputLogs(tempCanOutputLogs);
+}
+
+void BFGS::CalcDisplacement(double      *      *& matrixDisplacement,
+                            double const* const*  matrixOldCoordinates,
+                            const MolDS_base::Molecule& molecule)const{
+   //Calculate displacement (K_k at Eq. (15) in [SJTO_1983])
+   MallocerFreer::GetInstance()->Malloc(&matrixDisplacement, molecule.GetNumberAtoms(), CartesianType_end);
+   for(int i=0;i<molecule.GetNumberAtoms();i++){
+      const Atom*   atom = molecule.GetAtom(i);
+      const double* xyz  = atom->GetXyz();
+      for(int j=0;j<CartesianType_end;j++){
+         matrixDisplacement[i][j] = xyz[j] - matrixOldCoordinates[i][j];
+      }
+   }
+}
+
+void BFGS::StoreMolecularGeometry(double **& matrixCoordinates, 
+                                  const MolDS_base::Molecule& molecule)const{
+   //Store old coordinates
+   MallocerFreer::GetInstance()->Malloc(&matrixCoordinates, molecule.GetNumberAtoms(), CartesianType_end);
+   for(int i=0;i<molecule.GetNumberAtoms();i++){
+      const Atom*   atom = molecule.GetAtom(i);
+      const double* xyz  = atom->GetXyz();
+      for(int j=0;j<CartesianType_end;j++){
+         matrixCoordinates[i][j] = xyz[j];
+      }
+   }
+}
 }
